# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os.path

import gtk
import gobject
from kiwi.utils import gsignal, type_register

from gazpacho import gapi, util
from gazpacho.commandmanager import command_manager
from gazpacho.context import Context
from gazpacho.filewriter import XMLWriter
from gazpacho.gadget import Gadget, load_gadget_from_widget
from gazpacho.loader.gazpacholoader import GazpachoObjectBuilder
from gazpacho.model import GazpachoModelManager
from gazpacho.placeholder import Placeholder
from gazpacho.signaleditor import SignalInfo
from gazpacho.sizegroup import GSizeGroup
from gazpacho.uimanager import GazpachoUIM
from gazpacho.widgets.base.box import CommandBoxDeletePlaceholder
from gazpacho.widgetregistry import widget_registry

class Project(gobject.GObject):
    """
    Project is an object representing a specific user interface which
    can be saved on disk.

    Signals:
      changed: something in the project changes, used to set the save button
        insensitive for example
      add-gadget: a gadget was added
      remove-gadget: a gadget was removed
      gadget-name-changed: the name of a gadget changed, different from
        when the name in a widget changes, because in some cases the widget
        itself changes in a gadget
      add-action: an action or actiongroup was added
      remove-action: an action or actiongroup was removed
      action-name-changed: the name of an action changed
      add-sizegroup: a sizegroup was added
      remove-sizegroup: a sizegroup was removed

    @ivar name: The name of the project like network-conf
    @ivar path: The full path of the xml file for this project
    @ivar sizegroups: # A list of GSizeGroups that are part of this project
    @ivar selection: We need to keep the selection in the project because
      we have multiple projects and when the user switchs between them, he
      will probably not want to loose the selection. This is a list of
      widget items.
    @ivar uim: There is a UIManager in each project which holds the information
      about menus and toolbars
    @ivar model_manager: The model manager takes care of all the data models
      for TreeViews and ComboBoxes, ...
    @ivar undo_stack: Contains the undo (and redo) commands
    @ivar context: out context
    """
    gsignal('changed'),
    gsignal('add-gadget', object),
    gsignal('remove-gadget', object),
    gsignal('gadget-name-changed', object),
    gsignal('add-action', object),
    gsignal('remove-action', object),
    gsignal('action-name-changed', object),
    gsignal('add-sizegroup', object),
    gsignal('remove-sizegroup', object),

    def __init__(self, app):
        gobject.GObject.__init__(self)

        self._app = app
        self._changed = False
        self._domain = ''
        self._unsupported_widgets = {}
        self._version = 'libglade'
        self._widget_old_names = {} # widget -> old name of the
        self._widgets = {}

        self.name = None
        self.path = None
        self.sizegroups = []
        self.selection = WidgetSelection()
        self.tooltips = gtk.Tooltips()
        self.uim = GazpachoUIM(self)
        self.model_manager = GazpachoModelManager()
        self.undo_stack = UndoRedoStack()
        self.undo_stack.connect('changed', self._undo_stack_changed_cb)
        self.context = Context(self)

    # Private

    def _read_from_loader(self, loader):
        """
        Reads all data from the loader, such as widgets, ui manager
        definitions, signals.

        @param loader: the loader
        @type loader: L{gazpacho.loader.loader.ObjectBuilder} instance
        """
        self._domain = loader.get_domain() or ''
        self._version = loader.get_version()

        # Load UIM manager
        # Do this first since the custom menubar and toolbars adapters
        # depends on all ui definitions being loaded
        self.uim.load(loader)

        # Load models before widgets
        models = [w for w in loader.toplevels if isinstance(w, gtk.ListStore)]
        for model in models:
            self.model_manager.load_model(model)

        # Load the widgets
        for widget in loader.toplevels:
            if isinstance(widget, gtk.Widget):
                self._load_widget(widget)

        # Load sizegroups, must be done after loading all the widgets,
        # since the sizegroups has references to the widgets
        for sizegroup in loader.sizegroups:
            name = sizegroup.get_data('gazpacho::object-id')
            widgets =  sizegroup.get_data('gazpacho::sizegroup-widgets') or []
            gadgets = [Gadget.from_widget(widget)
                       for widget in widgets]
            self.add_sizegroup(GSizeGroup(name, sizegroup, gadgets))

        # Signals
        for signal in loader.get_signals():
            gobj, signal_name, signal_handler, signal_after = signal[:4]
            gadget = Gadget.from_widget(gobj)
            if gadget is None:
                continue
            gadget.add_signal_handler(SignalInfo(name=signal_name,
                                                 handler=signal_handler,
                                                 after=signal_after))

        self._unsupported_widgets = loader.get_unsupported_widgets()

        self.changed = False

    def _load_widget(self, widget):
        gadget = load_gadget_from_widget(widget, self)
        self.add_widget(widget)
        return gadget

    def _undo_stack_changed_cb(self, stack):
        """Callback for the undo stack's 'changed' signal."""
        self.changed = not self.undo_stack.on_saved()

    def _add_action_cb(self, action_group, action):
        self.emit('add-action', action)

    def _remove_action_cb(self, action_group, action):
        self.emit('remove-action', action)

    def _on_widget_notify_name(self, widget, pspec):
        old_name = self._widget_old_names[widget]
        new_name = widget.get_name()
        if old_name == new_name:
            # This happens but I'm not sure why, it would be nice if
            # it could be avoided
            return

        del self._widget_old_names[widget]
        del self._widgets[old_name]

        gadget = Gadget.from_widget(widget)
        if isinstance(widget, (gtk.MenuBar, gtk.Toolbar)):
            self.uim.update_widget_name(gadget)
            # gadget.widget might have been replaced with a new widget
            # after calling this method
            widget = gadget.widget
            widget.connect('notify::name', self._on_widget_notify_name)

        self._widget_old_names[widget] = new_name
        self._widgets[widget.get_name()] = widget

        self.emit('gadget-name-changed', gadget)

    # Properties

    def _get_changed(self):
        return self._changed

    def _set_changed(self, value):
        if self._changed != value:
            self._changed = value
            self.emit('changed')
    changed = property(_get_changed, _set_changed)

    # Public API

    def load(self, filename):
        """
        Loads a glade file from a filename and inserts the content of
        the file in a project.

        @param filename: glade file to open
        """
        loader = GazpachoObjectBuilder(filename=filename, app=self._app)
        self._read_from_loader(loader)
        self.path = filename
        self.name = os.path.basename(filename)

    def load_from_buffer(self, buffer):
        """
        Loads a glade buffer and inserts the content of the file in a project.

        @param filename: string
        """
        loader = GazpachoObjectBuilder(buffer=buffer, app=self._app)
        self._read_from_loader(loader)

    def get_domain(self):
        """
        Retrieves the translatable domain

        @returns: the domain
        """
        return self._domain

    def set_domain(self, domain):
        """
        Set the translatable domain for this glade file

        @param domain: domain to set
        @type domain: string
        """

        self._domain = domain

        self.changed = True

    def get_version(self):
        """
        Retrieves the version, possible values are currently
        'libglade' or 'gtkbuilder'

        @returns: the version
        """
        return self._version

    def get_unsupported_widgets(self):
        """
        Get a dict of the widgets that could not be loaded because
        they're not supported. The dict maps the class of the widget
        to a list of widget ids.

        @return: the unsupported widgets
        @rtype: dict
        """
        return self._unsupported_widgets

    def get_app(self):
        return self._app

    def get_id(self):
        return "project%d" % id(self)

    def get_widgets(self):
        return list(self._widgets.values())

    def get_gadget_by_name(self, name):
        if not name in self._widgets:
            return

        return Gadget.from_widget(self._widgets[name])

    def get_windows(self):
        return [widget for widget in list(self._widgets.values())
                           if widget.flags() & gtk.TOPLEVEL]

    def add_widget(self, widget, new_name=False):
        """
        Adds a widget to the project and optionally sets a new name
        if it already exists. Will emit add-gadget signal when done.

        @param widget:
        @param new_name: If True assign a new name if it already
          exists
        """
        # we don't list placeholders
        if isinstance(widget, Placeholder):
            return

        # The internal widgets (e.g. the label of a GtkButton) are handled
        # by gtk and don't have an associated gadget: we don't want to
        # add these to our list. It would be nicer to have a flag to check
        # (as we do for placeholders) instead of checking for the associated
        # gadget, so that we can assert that if a widget is _not_
        # internal, it _must_ have a corresponding gadget...
        # Anyway this suffice for now.
        gadget = Gadget.from_widget(widget)
        if not gadget:
            return

        children = gadget.get_children()

        gadget.project = self
        self._widget_old_names[widget] = widget.get_name()

        widget_name = widget.name
        if new_name and gadget.name in self._widgets:
            widget_name = self.set_new_widget_name(gadget.widget)

        widget.connect('notify::name', self._on_widget_notify_name)
        self._widgets[widget_name] = widget

        for child in children:
            self.add_widget(child, new_name=new_name)

        self.emit('add-gadget', gadget)

    def add_hidden_widget(self, widget):
        """
        Adds a hidden widget, eg one which is not displayed to a project.
        It also updates the name of the widget
        @param widget: the widget
        """
        if not isinstance(widget, gtk.Widget):
            raise TypeError("widget %r is not gtk.Widget instance" % widget)

        if isinstance(widget, gtk.Container):
            for child in util.get_all_children(widget):
                self.add_hidden_widget(child)

        self._widgets[widget.get_name()] = widget

    def remove_hidden_widget(self, widget):
        if not isinstance(widget, gtk.Widget):
            raise TypeError("widget %r is not gtk.Widget instance" % widget)

        if isinstance(widget, gtk.Container):
            for child in util.get_all_children(widget):
                self.remove_hidden_widget(child)

        if widget.get_name() in list(self._widgets.keys()):
            del self._widgets[widget.get_name()]

    def remove_widget(self, widget):
        if isinstance(widget, Placeholder):
            return

        gadget = Gadget.from_widget(widget)
        if not gadget:
            return

        for child in gadget.get_children():
            self.remove_widget(child)

        if widget in self.selection:
            self.selection.remove(widget, False)
            self.selection.selection_changed()

        del self._widgets[widget.get_name()]

        self.emit('remove-gadget', gadget)

    def delete_selection(self):
        """Delete the widget in the selection calling the right command"""
        assert len(self.selection) == 1

        widget = self.selection[0]
        gadget = Gadget.from_widget(widget)
        if gadget:
            gapi.delete_gadget(self, gadget)

        elif (isinstance(widget, Placeholder) and widget.is_deletable()):
            if len(widget.get_parent().get_children()) > 1:
                cmd = CommandBoxDeletePlaceholder(widget)
                command_manager.execute(cmd, self)

    # XXX: replace_widget
    def replace(self, widget_name, new_widget):
        """
        Replace an existing widget with a new one
        @param widget_name: name of widget to replace
        @param new_widget: new widget
        """

        widgets = self._widgets
        if not widget_name in widgets:
            raise KeyError("No widget called: %s" % widget_name)

        if not isinstance(new_widget, gtk.Widget):
            raise TypeError("new_widget must be a gtk.Widget")

        del widgets[widget_name]
        widgets[widget_name] = new_widget
        self.changed = True

    def add_sizegroup(self, sizegroup):
        """
        Add a SizeGroup to the project.

        @param sizegroup: the sizegroup to add
        @type sizegroup: gazpacho.sizegroup.GSizeGroup
        """
        if sizegroup in self.sizegroups:
            return

        self.sizegroups.append(sizegroup)
        self.emit('add-sizegroup', sizegroup)

    def remove_sizegroup(self, sizegroup):
        """
        Remove a SizeGroup from the project. Note that this doesn't
        remove the widgets from the sizegroup.

        @param sizegroup: the sizegroup to remove
        @type sizegroup: gazpacho.sizegroup.GSizeGroup
        """
        self.sizegroups.remove(sizegroup)
        self.emit('remove-sizegroup', sizegroup)

    def add_action_group(self, action_group):
        self.uim.add_action_group(action_group)

        self.emit('add-action', action_group)

    def remove_action_group(self, action_group):
        self.uim.remove_action_group(action_group)
        self.emit('remove-action', action_group)

    def change_action_name(self, action):
        self.emit('action-name-changed', action)

    def set_new_widget_name(self, widget):
        """Create a new name based on type."""
        if not isinstance(widget, gobject.GObject):
            raise TypeError("Instance must be a GObject, not %r" % widget)

        adapter = widget_registry.get_by_type(widget)
        base_name = adapter.generic_name
        i = 0
        while True:
            i += 1
            name = '%s%d' % (base_name, i)
            if not name in self._widgets:
                break

        widget.set_name(name)
        return name

    def save(self, path):
        xw = XMLWriter(project=self)
        widgets = list(self._widgets.values())
        widgets.sort(lambda x, y: cmp(x.name, y.name))
        models = self.model_manager.get_models()
        retval = xw.write(path, widgets, self.sizegroups, models, self.uim,
                          self._domain, self._version)

        self.undo_stack.mark_saved()
        self.path = path
        self.name = os.path.basename(path)

        self.changed = False

        return retval

    def serialize(self):
        xw = XMLWriter()
        widgets = list(self._widgets.values())
        widgets.sort(lambda x, y: cmp(x.name, y.name))
        models = self.model_manager.get_models()
        return xw.serialize_widgets(widgets, self.sizegroups, models, self.uim)

type_register(Project)


class UndoRedoStack(gobject.GObject):
    """Class representing an undo/redo stack."""

    gsignal('changed')

    def __init__(self):
        gobject.GObject.__init__(self)

        #  A stack with the last executed commands
        self._commands = []

        #  Points to current undo command
        self._current_undo_command = -1

        # The possition marked when the project was last saved
        self._saved_position = -1

    def mark_saved(self):
        """Mark that the project has been saved at the current
        possition.
        """
        self._saved_position = self._current_undo_command

    def on_saved(self):
        """Check whether we are on the possition that has been marked
        as changed or not.

        @return: True if we're on the saved possition otherwise False
        @rtype: bool
        """
        return self._saved_position == self._current_undo_command

    def has_undo(self):
        """Check if there are any commands in the undo stack."""
        return self._current_undo_command != -1

    def has_redo(self):
        """Check if there are any commands in the redo stack."""
        return self._current_undo_command + 1 != len(self._commands)

    def get_undo_info(self):
        """Get the current undo command description."""
        if self.has_undo():
            return self._commands[self._current_undo_command].description
        return None

    def get_redo_info(self):
        """Get the current redo command description."""
        if self.has_redo():
            return self._commands[self._current_undo_command + 1].description
        return None

    def pop_undo(self):
        """Return the current undo command and move it to the redo stack."""
        if self.has_undo():
            cmd = self._commands[self._current_undo_command]
            self._current_undo_command -= 1
            self.emit('changed')
            return cmd
        return None

    def pop_redo(self):
        """Return the current redo command and move it to the undo stack."""
        if self.has_redo():
            cmd =  self._commands[self._current_undo_command + 1]
            self._current_undo_command += 1
            self.emit('changed')
            return cmd
        return None

    def push_undo(self, command):
        """Put the command in the project command stack
        It tries to collapse the command with the previous command
        """
        # If there are no "redo" items, and the last "undo" item unifies with
        # us, then we collapse the two items in one and we're done
        if self.has_undo() and not self.has_redo():
            cmd = self._commands[self._current_undo_command]
            if not self.on_saved() and cmd.unifies(command):
                cmd.collapse(command)
                self.emit('changed')
                return

        # We should now free all the 'redo' items
        self._commands = self._commands[:self._current_undo_command + 1]

        # and then push the new undo item
        self._commands.append(command)
        self._current_undo_command += 1

        self.emit('changed')

    def get_undo_commands(self):
        """Get a list of all undo commands."""
        return self._commands[:self._current_undo_command + 1]

    def get_redo_commands(self):
        """Get a list of all redo commands."""
        return self._commands[self._current_undo_command + 1:]

type_register(UndoRedoStack)

def refresh_selected_nodes(widget):
    """ Invalidates the portion of the window covered by the selection
    nodes for the given widget """

    if not widget.window:
        return

    if widget.get_parent():
        window = widget.get_parent_window()
    else:
        window = widget.window
    window.invalidate_rect(widget.allocation, True)

class WidgetSelection(gobject.GObject):
    """
    The widget selection keeps track of which widgets are currently
    selected and provide means to manipulate the selection.

    When the selection is modified a 'selection-changed' signal will
    be emitted.
    """

    gsignal('selection-changed')

    def __init__(self):
        gobject.GObject.__init__(self)
        self._selection = []

    def selection_changed(self):
        """
        Emit a selection-changed signal.
        """
        self.emit('selection_changed')

    def clear(self, emit_signal=True):
        """
        Clear the selection. This might emit a selection-changed
        signal.

        @param emit_signal: if the signal should be emitted
        @type emit_signal: bool
        """
        if not self._selection:
            return

        for widget in self._selection:
            refresh_selected_nodes(widget)

        self._selection = []

        if emit_signal:
            self.selection_changed()

    def remove(self, widget, emit_signal=True):
        """
        Remove the widget from the current selection. This might emit
        a selection-changed signal.

        @param widget: the widget to remove
        @type widget: gtk.Widget
        @param emit_signal: if the signal should be emitted
        @type emit_signal: bool
        """
        if widget not in self._selection:
            return

        refresh_selected_nodes(widget)

        self._selection.remove(widget)
        if emit_signal:
            self.selection_changed()

    def add(self, widget, emit_signal=True):
        """
        Add the widget to the selection. This might emit a
        selection-changed signal.

        @param widget: the widget to add
        @type widget: gtk.Widget
        @param emit_signal: if the signal should be emitted
        @type emit_signal: bool
        """
        if widget in self._selection:
            return

        self._selection.insert(0, widget)
        if emit_signal:
            self.selection_changed()

        refresh_selected_nodes(widget)

    def set(self, widget, emit_signal=True):
        """
        Clear the selection and add the specified widget. This might
        emit a selection-changed signal.

        @param widget: the widget to add
        @type widget: gtk.Widget
        @param emit_signal: if the signal should be emitted
        @type emit_signal: bool
        """
        self.clear(False)
        self.add(widget, emit_signal)

    def toggle(self, widget, emit_signal=True):
        """
        The the widget is in the selection it will be removed
        otherwise it will be added.

        @param widget: the widget to add or remove
        @type widget: gtk.Widget
        @param emit_signal: if the signal should be emitted
        @type emit_signal: bool
        """
        if widget in self._selection:
            self.remove(widget, emit_signal)
        else:
            self.add(widget, emit_signal)

    def circle(self, leaf_widget):
        """
        Select the parent of the selected widget. If the selected
        widget is a toplevel widget or there isn't a selected widget
        in the current branch of the tree, select the leaf widget.

        @param leaf_widget: the leaf widget
        @type leaf_widget: gtk.Widget
        """
        parent_gadget = util.get_parent(leaf_widget)

        # If the leaf widget doesn't have a parent select it.
        if not parent_gadget:
            self.set(leaf_widget)
            return

        # If there is no or multiple selections, select the leaf
        # widget. Note that it's not sure that the leaf widget has a
        # gadget (if it' a placeholder).
        if len(self._selection) != 1:
            self.set(leaf_widget)
            return

        selected = self._selection[0]

        # If the leaf widget is selected, select its parent
        if leaf_widget is selected:
            self.set(parent_gadget.widget)
            return

        # Circle through the tree
        new_selection = leaf_widget
        while parent_gadget:
            if parent_gadget.widget is selected:
                # Choose the leaf if selected is toplevel
                if parent_gadget.is_toplevel():
                    new_selection = leaf_widget
                # otherwise choose the parent
                else:
                    new_selection = parent_gadget.get_parent().widget
                break
            parent_gadget = parent_gadget.get_parent()
        self.set(new_selection)


    def __iter__(self):
        return iter(self._selection)

    def __len__(self):
        return len(self._selection)

    def __contains__(self, item):
        return item in self._selection

    def __getitem__(self, key):
        return self._selection[key]

type_register(WidgetSelection)



from django.contrib.contenttypes.models import ContentType
from south.logger import get_logger
from south.v2 import DataMigration

# Helper class for (0012|0002)_update_content_types
class UpdateContentTypeMigration(DataMigration):
    _app_label = None

    def __init__(self, *args, **kwargs):
        super(UpdateContentTypeMigration, self).__init__(*args, **kwargs)

        # find a list of models used in this product module from the frozen orm
        if not self._app_label:
            return

        models = []
        for model in self.models.keys():
            try:
                i = model.index(self._app_label)
                models.append(model[len(self._app_label)+1:])
            except ValueError:
                continue
        self._models = models

    def migrate_contenttype(self, from_app, to_app, models=None):
        models = models or self._models

        # ideally, we should have frozen content types too, but we're lazy.
        q = ContentType.objects.filter(
            app_label=from_app,
            model__in=models,
        )

        # sanity check; just warn, don't have to error out
        if not len(q) == len(models):
            get_logger().warning(
                "Not all content types for models (%s) in app %s were found" % \
                    (models, from_app))

        for ct in q:
            ct.app_label = to_app
            ct.save()
            get_logger().info("Updated content type for model %s (ID: %s)" % (ct.model, ct.id))

    def forwards(self, orm):
        self.migrate_contenttype('product', self._app_label)

    def backwards(self, orm):
        self.migrate_contenttype(self._app_label, 'product')



#!/usr/bin/env python
import os.path
import sys

DIRNAME = os.path.dirname(__file__)
# trick to get the two-levels up directory, which for the "simple" project should be the satchmo dir
_parent = lambda x: os.path.normpath(os.path.join(x, '..'))

SATCHMO_DIRNAME = _parent(_parent(DIRNAME))
SATCHMO_APPS = os.path.join(SATCHMO_DIRNAME, 'apps')

if not SATCHMO_APPS in sys.path:
    sys.path.append(SATCHMO_APPS)

if not DIRNAME in sys.path:
    sys.path.append(DIRNAME)

from django.core.management import execute_manager
try:
    import settings # Assumed to be in the same directory.
except ImportError:
    import sys
    sys.stderr.write("Error: Can't find the file 'settings.py' in the directory containing %r. It appears you've customized things.\nYou'll have to run django-admin.py, passing it your settings module.\n(If the file settings.py does indeed exist, it's causing an ImportError somehow.)\n" % __file__)
    sys.exit(1)

if __name__ == "__main__":
    execute_manager(settings)



import sys

from test_support import WSGIControllerTestCase

class WSGIAppTestCase(WSGIControllerTestCase):

    def test_render_index(self):
        response = self.client.get('/') # /blog/ 
        self.assertEqual(response[1], '200 OK')




# Copyright (C) 2005 by Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from kiwi.component import Attribute, Interface

class BaseWidgetAdaptor:
    name = None
    type = None

class BaseLibrary:
    def __init__(self, name, library_name):
        pass

    def create_widget(self, gtype):
        pass

class IGazpachoApp(Interface):
    """Provides a gazpacho application"""

    add_class = Attribute('add_class')

    def get_current_project(self):
        pass

    def get_window(self):
        pass


class IReferencable(Interface):
    """This interface provides means for clearing and restoring
    references to a gadget.

    Every deletable object that will keep a reference to a gadget
    should implement this interface. The object implementing this
    interface will also be responsible for adding and removing itself
    to the reference container for the gadget to which it refers.
    """

    def remove_reference(gadget):
        """
        This method will be called by the reference manager when the
        gadget has been removed or deleted.
        """
        pass

    def add_reference(gadget):
        """
        This method will be called by the reference manager when a
        deleted gadget has been restored.
        """
        pass


class IPluginManager(Interface):
    """Manages a set of plugins

    Takes care of loading the plugins and activating/deactivating
    them.

    It search for plugins in $datadir/plugins and also in the user home
    directory (~/.gazpacho/plugins). The later location takes preference
    over the system wide plugin directory.

    Every plugin should be a python package with a .desktop file containing
    some metadata. Check the PluginInfo class for information about that
    metadata
    """

    def load_plugins():
        """Load plugins in the standard plugins directories"""

    def load_plugins_dir(dirname):
        """Load plugins in the specified directory"""

    def get_plugins():
        """Returns a list of PluginInfo objects that represents the
        loaded plugins"""

    def is_activated(plugin_name):
        """True if a plugin with that name is activated"""

    def activate_plugins(plugin_names):
        """Activate all the plugins in the list plugin_names"""

    def activate_plugin(plugin_name):
        """Activate just the plugin with the name 'plugin_name'"""

    def deactivate_all():
        """Deactivate all plugins that are activated"""

    def deactivate_plugin(plugin):
        """Deactivate just the plugin passed as the argument.

        The argument can be a plugin name or a Plugin instance
        """



# Copyright (C) 2005 by Sicem S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import sys
import urllib.request, urllib.parse, urllib.error

import gtk

from gazpacho import __version__
from gazpacho.config import config
from gazpacho.i18n import _

class BugReportDialog(gtk.Dialog):
    def __init__(self, parent=None):
        gtk.Dialog.__init__(self, 'Send Bug report', parent,
                            gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR,
                            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                             gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        self.set_border_width(6)
        # 304 = 430 / sqrt(2) = golden number
        self.set_default_size(430, 304)
        self.vbox.set_spacing(12)

        text1 = _('What were you doing before the error occured?')
        frame1 = self._create_frame(text1)
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.set_shadow_type(gtk.SHADOW_IN)
        self.description_textview = gtk.TextView()
        sw.add(self.description_textview)
        frame1.add(sw)
        self.vbox.pack_start(frame1, True, True)

        text2 = _('Personal information (optional)') + ':'
        frame2 = self._create_frame(text2)
        table = gtk.Table(2, 2)
        table.set_border_width(6)
        table.set_row_spacings(6)
        table.set_col_spacings(6)
        label1 = gtk.Label(_('Name') + ':')
        self.name_entry = gtk.Entry()
        self.name_entry.set_text(config.get_bugname())
        label2 = gtk.Label(_('E-Mail') + ':')
        self.email_entry = gtk.Entry()
        self.email_entry.set_text(config.get_bugmail())
        table.attach(label1, 0, 1, 0, 1, 0, 0)
        table.attach(self.name_entry, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, 0)
        table.attach(label2, 0, 1, 1, 2, 0, 0)
        table.attach(self.email_entry, 1, 2, 1, 2, gtk.EXPAND|gtk.FILL, 0)
        frame2.add(table)
        self.vbox.pack_start(frame2, False, False)

        self.vbox.show_all()

    def _create_frame(self, labeltext):
        frame = gtk.Frame()
        frame.set_border_width(6)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        label = gtk.Label("<b>%s</b>" % labeltext)
        label.set_use_markup(True)
        frame.set_label_widget(label)
        return frame

    def get_description(self):
        buffer = self.description_textview.get_buffer()
        return buffer.get_text(*buffer.get_bounds())

    def get_name(self):
        return self.name_entry.get_text()

    def get_email(self):
        return self.email_entry.get_text()

def get_revision():
    basedir = os.path.join(os.path.dirname(__file__), '..')
    svn = os.path.join(basedir, '.svn')
    if not os.path.exists(svn):
        return '$Revision: 2415 $'[11:-2]

    max = 0
    for directory in ['.', 'gazpacho', 'gazpacho/app']:
        entries = os.path.join(basedir, directory, '.svn', 'entries')
        try:
            lines = open(entries).readlines()
        except (OSError, IOError):
            continue
        for line in lines:
            if 'committed-rev' in line:
                current = int(line.split('"', 2)[1])
                if current > max:
                    max = current

    return max


BUG_TEMPLATE = """Package: Gazpacho
Version: %(version)s (r%(revision)s)
Synopsis: %(subject)s
Bugzilla-Product: Gazpacho
Bugzilla-Component: general

Date: %(date)s

User name: %(username)s %(email)s

The user was doing the following task:
%(description)s

Exception traceback:
%(traceback)s

Python version: %(python_version)s
GTK+ version: %(gtk_version)s
PyGTK version: %(pygtk_version)s

Platform: %(platform)s"""


def send_bugreport(tb, exception_text, parent=None):
    """
    Send a bugreport to http://bugzilla.gnome.org in the product
    name Gazpacho.

    Uses an http proxy at http://gazpacho.sicem.biz/bugproxy because
    not all the users has a local smtp available, specially win32 users.

    Return True if the bug was sent or False otherwise
    """

    d = BugReportDialog(parent)
    r = d.run()

    if r != gtk.RESPONSE_ACCEPT:
        d.destroy()
        return False

    import time
    date = time.strftime("%Y-%m-%d %H:%M")
    filename = os.path.basename(tb.tb_frame.f_code.co_filename)
    for lastline in exception_text.split('\n')[::-1]:
        if lastline != '':
            break

    subject = '%s:%d %s' % (filename, tb.tb_lineno, lastline)
    email = d.get_email()
    if email:
        email = '(%s)' % email

    bug_info = dict(description=d.get_description() or 'Unknown',
                    traceback=exception_text or 'No traceback available',
                    username=d.get_name() or 'Unknown',
                    email=email,
                    date=date,
                    subject=subject,
                    platform=sys.platform,
                    python_version=sys.version,
                    version=__version__,
                    gtk_version=gtk.gtk_version,
                    pygtk_version=gtk.pygtk_version,
                    revision=get_revision())

    params = dict(subject=subject, body=BUG_TEMPLATE % bug_info)
    params['from'] = d.get_email() or 'lgs@sicem.biz'

    # send the report via HTTP POST
    data = urllib.parse.urlencode(params)
    f = urllib.request.urlopen("http://gazpacho.sicem.biz/bugproxy", data)
    print(repr(f.read()))

    config.set_bugname(d.get_name())
    config.set_bugmail(d.get_email())
    config.save()

    d.destroy()
    return True

if __name__ == '__main__':
    print(get_revision())



# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk
from kiwi.ui.dialogs import BaseDialog

from gazpacho.i18n import _

class CloseConfirmationDialog(BaseDialog):
    """A base class for the close confirmation dialogs. It lets the
    user choose to Save the projects, Close without saving them and to
    abort the close operation.
    """

    def __init__(self, projects, toplevel=None):
        BaseDialog.__init__(self, parent=toplevel)

        self.add_buttons(_("Close _without Saving"), gtk.RESPONSE_NO,
                         gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                         gtk.STOCK_SAVE, gtk.RESPONSE_YES)

        save, cancel, close = self.action_area.get_children()
        if gtk.settings_get_default().get_property(
            "gtk-alternative-button-order"):
            save, close = close, save
        save.set_name("save")
        cancel.set_name("cancel")
        close.set_name("close")

        self.set_default_response(gtk.RESPONSE_YES)

        self.set_border_width(6)
        self.set_resizable(False)
        self.set_has_separator(False)
        self.vbox.set_spacing(12)

        # Add the warning image to the dialog
        warning_image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
                                                 gtk.ICON_SIZE_DIALOG)
        warning_image.set_alignment(0.5, 0)

        self._hbox = gtk.HBox(False, 12)
        self._hbox.set_border_width(6)
        self._hbox.pack_start(warning_image, False, False, 0)

        self.vbox.pack_start(self._hbox)


class SingleCloseConfirmationDialog(CloseConfirmationDialog):
    """A close confirmation dialog for a single project."""

    def __init__(self, project, toplevel=None):
        CloseConfirmationDialog.__init__(self, toplevel)
        self._project = project

        submsg1 = _('Save changes to project "%s" before closing?') % \
                  project.name
        submsg2 = _("Your changes will be lost if you don't save them.")
        msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
              (submsg1, submsg2)
        label = gtk.Label(msg)
        label.set_use_markup(True)
        label.set_line_wrap(True)
        label.set_alignment(0.0, 0.5)
        self._hbox.pack_start(label)

        self.vbox.show_all()

    def get_projects(self):
        """Get a list of the projects that should be saved."""
        return [self._project]

class MultipleCloseConfirmationDialog(CloseConfirmationDialog):
    """A close confirmation dialog for a multiple project. It presents
    a list of the projects and let the user choose which to save."""

    def __init__(self, projects, toplevel=None):
        CloseConfirmationDialog.__init__(self, toplevel)
        self._projects = projects
        self._projects.sort(lambda x, y: cmp(x.name, y.name))

        # ListStore (boolean, Project) storing information about which
        # projects should be saved.
        self._model = None

        submsg1 = ('There are %d projects with unsaved changes. '
                   'Do you want to save those changes before closing?') % \
                 len(projects)
        submsg2 = _("Your changes will be lost if you don't save them.")
        msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
              (submsg1, submsg2)
        label = gtk.Label(msg)
        label.set_use_markup(True)
        label.set_line_wrap(True)
        label.set_alignment(0.0, 0.5)

        list_label = gtk.Label()
        list_label.set_markup_with_mnemonic(_("S_elect the projects you "
                                              "want to save:"))
        list_label.set_line_wrap(True)
        list_label.set_alignment(0.0, 0.5)
        list_label.set_padding(0, 6)

        view = self._create_project_list()
        list_label.set_mnemonic_widget(view)

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.add_with_viewport(view)
        scrolled_window.set_size_request(200, 100)

        project_vbox = gtk.VBox(False)
        project_vbox.pack_start(label)
        project_vbox.pack_start(list_label)
        project_vbox.pack_start(scrolled_window)
        self._hbox.pack_start(project_vbox)

        self.vbox.show_all()

    def _create_project_list(self):
        """Create, populate and return a TreeView containing the
        unsaved projects."""
        # Create a ListStore model
        self._model = gtk.ListStore(bool, object)
        for project in self._projects:
            self._model.append([True, project])

        # Create the TreeView
        view = gtk.TreeView(self._model)
        view.set_headers_visible(False)

        # Create the check-box column
        toggle_renderer = gtk.CellRendererToggle()
        toggle_renderer.set_property('activatable', True)
        toggle_renderer.connect("toggled", self._toggled_cb, (self._model, 0))
        toggle_column = gtk.TreeViewColumn('Save', toggle_renderer)
        toggle_column.add_attribute(toggle_renderer, 'active', 0)
        view.append_column(toggle_column)

        # Create the project column
        def render_func(treeviewcolumn, renderer, model, iter):
            project = model[iter][1]
            renderer.set_property('text', project.name)
            return
        text_renderer = gtk.CellRendererText()
        text_column = gtk.TreeViewColumn('Project', text_renderer)
        text_column.set_cell_data_func(text_renderer, render_func)
        view.append_column(text_column)

        return view

    def _toggled_cb(self, renderer, path, user_data):
        """Callback to change the state of the check-box."""
        model, column = user_data
        model[path][column] = not model[path][column]

    def get_projects(self):
        """Get a list of the projects that should be saved."""
        return [project for saved, project in self._model if saved]

class UnsupportedWidgetsDialog(gtk.Dialog):
    """
    Dialog used to display informatin about widgets that are not
    supported by Gazpacho.
    """

    def __init__(self, window, widgets):
        """
        Initialize the dialog.

        @param widgets: dict mapping widget class to a list of widget ids
        @type: widgets: dict (type: list (of str))
        """
        gtk.Dialog.__init__(self,
                            title='',
                            parent=window,
                            flags=gtk.DIALOG_DESTROY_WITH_PARENT | \
                            gtk.DIALOG_NO_SEPARATOR,
                            buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))

        self.set_border_width(12)
        self.vbox.set_spacing(6)

        msg = _('Unsupported widgets')
        label = gtk.Label('<b><big>%s</big></b>' % msg)
        label.set_use_markup(True)
        self.vbox.pack_start(label, False)

        msg = _("The following widgets could not be loaded."
                " Please note that if you save this file"
                " these widgets will be lost.")
        label = gtk.Label('<small>%s</small>' % msg)
        label.set_line_wrap(True)
        label.set_use_markup(True)
        self.vbox.pack_start(label, False)

        msg = self._create_widget_message(widgets)
        label = gtk.Label(msg)
        self.vbox.pack_start(label, False)

        self.vbox.show_all()

    def _create_widget_message(self, widgets):
        msg_list = []
        for widget_class, widget_ids in widgets.items():
            for widget_id in widget_ids:
                msg_list.append('%s (%s)' % (widget_class, widget_id))

        return '\n'.join(msg_list)



from django.core.paginator import Paginator, EmptyPage
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _
from livesettings import config_value


from satchmo_utils.views import bad_or_missing

def home(request, template="shop/index.html"):
    # Display the category, its child categories, and its products.
    
    from product.views import display_featured

    if request.method == "GET":
        currpage = request.GET.get('page', 1)
    else:
        currpage = 1
        
    featured = display_featured()
    
    count = config_value('PRODUCT','NUM_PAGINATED')
    
    paginator = Paginator(featured, count)
    
    is_paged = False
    page = None
    
    try:
        paginator.validate_number(currpage)
    except EmptyPage:
        return bad_or_missing(request, _("Invalid page number"))
            
    is_paged = paginator.num_pages > 1
    page = paginator.page(currpage)
        
    ctx = RequestContext(request, {
        'all_products_list' : page.object_list,        
        'is_paginated' : is_paged,
        'page_obj' : page,
        'paginator' : paginator
    })
    
    return render_to_response(template, context_instance=ctx)
    



import sandbox

from pkg_resources import resource_filename

from sqlalchemy import create_engine, MetaData
from sqlalchemy.orm import scoped_session, sessionmaker

from notmm.utils.django_settings import LazySettings
from notmm.utils.configparse import as_bool

# init the local application data store
app_conf = sandbox.global_conf['sandbox']

# init the settings module (thread-local)
settings = LazySettings()

# SQLAlchemy settings
backend = app_conf['backend']
schema = backend + ':///' + resource_filename('sandbox', app_conf['path'])

engine = create_engine(schema, echo=as_bool(app_conf['echo']))
metadata = MetaData(engine)
Session = scoped_session(sessionmaker(bind=engine))




#!/usr/bin/env python
#to create the database:
# schevo db create --app lib/site-packages/moviereviews zodb:///var/db/zodb/test.fs

from schevo import database
from ZEO import ClientStorage
from ZODB import DB
from test_support import unittest

class ClientStorageTestCase(unittest.TestCase):
    def setUp(self):
        #self.filename = 'zodb:///var/db/zodb/test.fs'
        self.addr = ('127.0.0.1', 4343)
    def test_open(self):
        self.storage = ClientStorage.ClientStorage(self.addr)
        self.zodb = DB(self.storage)
        self.conn = self.zodb.open()
        

class SchevoTestCase(unittest.TestCase):

    connected = False

    def setUp(self):
        self.dbname = 'zodb://127.0.0.1:4343'
        self.db = database.open(self.dbname)
    def tearDown(self):
        self.db.close()

    def test_open(self):
        #self.assertEqual(equivalent(db, db2), True)
        self.assertEqual(self.db != None, True)
    
    def test_extents(self):
        lst = self.db.extents()
        self.assertEqual(isinstance(lst, list), True) #print lst

    def test_populate(self):
        #self.db.populate()
        #print sorted(item for item in self.db.Message)
        pass



"""Several helper functions."""

import sys,time,urllib
#from notmm.release import BASEVERSION

try:
    import termcolor
    _colored = termcolor.colored
except (ImportError, AttributeError):
    from django.utils.termcolors import colorize as _colored
    
__all__ = ('stdout', 
           'colored', 
           'notification',
           'SimpleHTTPBanner', 
           'get_bind_addr', 
           'daemonize')


def stdout(s):
    return sys.stderr.write("%s\n"%s)

def colored(message, color='cyan', **kwargs):
    """Write a string to the terminal with colored output"""
    stdout(_colored(message, color, **kwargs))

_event_types = {
    'info': lambda s: colored(s, color='blue', attrs=['bold']),
    'debug': lambda s: colored(s, color='white', attrs=['bold']),
    'warning': lambda s: colored(s, color='orange', attrs=['bold'])
}
info = _event_types['info']
notification = info




from payment.modules.base import HeadlessPaymentProcessor

class PaymentProcessor(HeadlessPaymentProcessor):

    def __init__(self, settings):
        super(PaymentProcessor, self).__init__('sermepa', settings)



from datetime import datetime
from decimal import Decimal
from django.utils.translation import ugettext_lazy as _
from livesettings import config_get_group
from satchmo_store.shop.models import Order, OrderAuthorization, OrderPayment, OrderPaymentFailure, OrderPendingPayment, OrderStatus
import logging

log = logging.getLogger('payment.modules.base')

NOTSET = object()

class BasePaymentProcessor(object):

    def __init__(self, label, payment_module):
        self.key = payment_module.KEY.value
        self.settings = payment_module
        self.label = label
        self.log = logging.getLogger('payment.' + label)
        self.order = None

    def allowed(self, user, amount):
        """Allows different payment processors to be allowed for certain situations."""
        return True
        
    def authorize_and_release(self, order=None, amount=NOTSET, testing=False):
        if not order:
            order = self.order
        else:
            self.order = order
        if amount == NOTSET:
            amount = Decimal('0.01')
        self.log_extra('authorize_and_release for order #%i, %s', order.id, amount)
        result = self.authorize_payment(testing=testing, order=order, amount=amount)
        if result.success:
            auths = order.authorizations.filter(complete=False).order_by('-id')
            if auths.count() > 0:
                auth = auths[0]
            self.log_extra('releasing successful authorize_and_release for order #%i [%s], %s.  AUTH', 
                order.id,  amount, auth.transaction_id)
            return self.release_authorized_payment(order=order, auth=auth, testing=testing)
        else:
            self.log_extra('early authorization was not successful for: %s', order)
            return result

    def authorize_payment(self, testing=False, order=None, amount=NOTSET):
        """Authorize a single payment, must be overridden to function"""
        self.log.warn('Module does not implement authorize_payment: %s', self.key)
        return ProcessorResult(False, _("Not Implemented"))

    def can_authorize(self):
        return False

    def can_process(self):
        return True

    def can_refund(self):
        return False

    def can_recur_bill(self):
        return False

    def capture_authorized_payments(self, order=None):
        """Capture all outstanding payments for this processor.  This is usually called by a 
        listener which watches for a 'shipped' status change on the Order."""
        results = []
        if self.can_authorize():
            if not order:
                order = self.order
            auths = order.authorizations.filter(payment__exact=self.key, complete=False)
            self.log_extra('Capturing %i %s authorizations for order #%i', auths.count(), self.key, order.id)
            self.prepare_data(order)
            for auth in auths:
                results.append(self.capture_authorized_payment(auth))
                
        return results

    def capture_authorized_payment(self, authorization, testing=False, order=None, amount=NOTSET):
        """Capture a single payment, must be overridden to function"""
        self.log.warn('Module does not implement capture_payment: %s', self.key)
        return ProcessorResult(False, _("Not Implemented"))

    def capture_payment(self, testing=False, order=None, amount=NOTSET):
        """Capture payment without an authorization step.  Override this one."""
        self.log.warn('Module does not implement authorize_and_capture: %s', self.key)
        return ProcessorResult(False, _("Not Implemented"))
        
    def create_pending_payment(self, order=None, amount=NOTSET):
        if not order:
            order = self.order
        recorder = PaymentRecorder(order, self.settings)
        return recorder.create_pending(amount=amount)

    def get_recurring_orderitems(self):
        """Iterate through the order and get all recurring billing items"""
        subscriptions = []
        for orderitem in self.order.orderitem_set.all():
            product = orderitem.product
            if product.is_subscription:
                self.log_extra('Found subscription product: %s', product.slug)
                if product.subscriptionproduct.recurring:
                    self.log_extra('Subscription is recurring: %s', product.slug)
                    subscriptions.append(orderitem)
                elif product.subscriptionproduct.trial_set.count() > 0:
                    self.log_extra('Not recurring, but it has a trial: %s', product.slug)
                    subscriptions.append(orderitem)
                else:
                    self.log_extra('Not a recurring product: %s ', product.slug)
            else:
                self.log_extra('Not a subscription product: %s', product.slug)
        return subscriptions

    def is_live(self):
        return self.settings.LIVE.value

    def log_extra(self, msg, *args):
        """Send a log message if EXTRA_LOGGING is set in settings."""
        if self.settings.EXTRA_LOGGING.value:
            self.log.info("(Extra logging) " + msg, *args)

    def prepare_data(self, order):
        self.order = order

    def process(self, testing=False):
        """This will process the payment."""
        if self.can_authorize() and not self.settings.CAPTURE.value:
            self.log_extra('Authorizing payment on order #%i', self.order.id)
            return self.authorize_payment(testing=testing)
        else:
            self.log_extra('Capturing payment on order #%i', self.order.id)
            return self.capture_payment(testing=testing)
            
    def record_authorization(self, amount=NOTSET, transaction_id="", reason_code="", order=None):
        """
        Convert a pending payment into a real authorization.
        """
        if not order:
            order = self.order

        recorder = PaymentRecorder(order, self.settings)
        recorder.transaction_id = transaction_id
        recorder.reason_code = reason_code
        return recorder.authorize_payment(amount=amount)

    def record_failure(self, amount=NOTSET, transaction_id="", reason_code="", 
        authorization=None, order=None, details=""):
        """
        Add an OrderPaymentFailure record
        """
        log.debug('record_failure for %s', order)
        if not order:
            order = self.order

        recorder = PaymentRecorder(order, self.settings)
        recorder.transaction_id = transaction_id
        recorder.reason_code = reason_code
        recorder.record_failure(amount, details=details, authorization=authorization)

    def record_payment(self, amount=NOTSET, transaction_id="", reason_code="", authorization=None, order=None):
        """
        Convert a pending payment or an authorization.
        """
        if not order:
            order = self.order
        recorder = PaymentRecorder(order, self.settings)
        recorder.transaction_id = transaction_id
        recorder.reason_code = reason_code

        if authorization:
            payment = recorder.capture_authorized_payment(authorization, amount=amount)
            authorization.complete=True
            authorization.save()

        else:
            payment = recorder.capture_payment(amount=amount)

        return payment

    def release_authorized_payment(self, order=None, auth=None, testing=False):
        """Release a previously authorized payment."""
        self.log.warn('Module does not implement released_authorized_payment: %s', self.key)
        return ProcessorResult(False, _("Not Implemented"))

class HeadlessPaymentProcessor(BasePaymentProcessor):
    """A payment processor which doesn't actually do any processing directly.
    
    This is used for payment providers such as PayPal and Google, which are entirely
    view/form based.
    """
    
    def can_process(self):
        return False

class PaymentRecorder(object):
    """Manages proper recording of pending payments, payments, and authorizations."""
    
    def __init__(self, order, config):
        self.order = order
        self.key = unicode(config.KEY.value)
        self.config = config
        self._amount = NOTSET
        self.transaction_id = ""
        self.reason_code = ""
        self.orderpayment = None
        self.pending = None
        
    def _set_amount(self, amount):
        if amount != NOTSET:
            self._amount = amount

    def _get_amount(self):
        if self._amount == NOTSET:
            return self.order.balance
        else:
            return self._amount

    amount = property(fset=_set_amount, fget=_get_amount)

    def _get_pending(self):
        self.pendingpayments = self.order.pendingpayments.filter(payment__exact=self.key)
        if self.pendingpayments.count() > 0:
            self.pending = self.pendingpayments[0]
            log.debug("Found pending payment: %s", self.pending)
            
    def authorize_payment(self, amount=NOTSET):
        """Make an authorization, using the existing pending payment if found"""
        self.amount = amount
        log.debug("Recording %s authorization of %s for %s", self.key, self.amount, self.order)
        
        self._get_pending()

        if self.pending:
            self.orderpayment = OrderAuthorization()
            self.orderpayment.capture = self.pending.capture
            
            if amount == NOTSET:
                self.set_amount_from_pending()
            
        else:
            log.debug("No pending %s authorizations for %s", self.key, self.order)
            self.orderpayment = OrderAuthorization(
                order=self.order, 
                payment=self.key)

        self.cleanup()
        return self.orderpayment

    def capture_authorized_payment(self, authorization, amount=NOTSET):
        """Convert an authorization into a payment."""
        self.amount = amount
        log.debug("Recording %s capture of authorization #%i for %s", self.key, authorization.id, self.order)

        self.orderpayment = authorization.capture
        self.cleanup()
        return self.orderpayment

    def capture_payment(self, amount=NOTSET):
        """Make a direct payment without a prior authorization, using the existing pending payment if found."""
        self.amount = amount

        self._get_pending()
        
        if self.pending:
            self.orderpayment = self.pending.capture
            log.debug("Using linked payment: %s", self.orderpayment)

            if amount == NOTSET:
                self.set_amount_from_pending()

        else:
            log.debug("No pending %s payments for %s", self.key, self.order)
        
            self.orderpayment = OrderPayment(
                order=self.order, 
                payment=self.key)
                
        log.debug("Recorded %s payment of %s for %s", self.key, self.amount, self.order)
        self.cleanup()
        return self.orderpayment
        
    def record_failure(self, amount=NOTSET, details="", authorization=None):
        log.info('Recording a payment failure: order #%i, code %s\nmessage=%s', self.order.id, self.reason_code, details)
        self.amount = amount
            
        failure = OrderPaymentFailure.objects.create(order=self.order, 
            details=details, 
            transaction_id=self.transaction_id,
            amount = self.amount,
            payment = self.key,
            reason_code = self.reason_code
        )
        return failure
    
    def cleanup(self):
        if self.pending:
            pending = self.pending
            self.orderpayment.capture = pending.capture
            self.orderpayment.order = pending.order
            self.orderpayment.payment = pending.payment
            self.orderpayment.details = pending.details

            # delete any extra pending orderpayments
            for p in self.pendingpayments:
                if p != pending and p.capture.transaction_id=='LINKED':
                    p.capture.delete()
                p.delete()

        self.orderpayment.reason_code=self.reason_code
        self.orderpayment.transaction_id=self.transaction_id
        self.orderpayment.amount=self.amount

        self.orderpayment.time_stamp = datetime.now()
        self.orderpayment.save()

        order = self.orderpayment.order

        if order.paid_in_full:
            def _latest_status(order):
                    try:
                        curr_status = order.orderstatus_set.latest()
                        return curr_status.status
                    except OrderStatus.DoesNotExist:
                        return ''

            if _latest_status(order) in ('', 'New'):
                order.order_success()
                # order_success listeners or product methods could have modified the status. reload it.
                if _latest_status(order) == '':
                    order.add_status('New')
                
    def create_pending(self, amount=NOTSET):
        """Create a placeholder payment entry for the order.  
        This is done by step 2 of the payment process."""
        if amount == NOTSET:
            amount = Decimal("0.00")
            
        self.amount = amount

        self._get_pending()
        ct = self.pendingpayments.count()
        if ct > 0:
            log.debug("Deleting %i expired pending payment entries for order #%i", ct, self.order.id)

            for pending in self.pendingpayments:
                if pending.capture.transaction_id=='LINKED':
                    pending.capture.delete()
                pending.delete()
        
        log.debug("Creating pending %s payment of %s for %s", self.key, amount, self.order)

        self.pending = OrderPendingPayment.objects.create(order=self.order, amount=amount, payment=self.key)
        return self.pending

    def set_amount_from_pending(self):
        """Try to figure out how much to charge. If it is set on the "pending" charge use that
        otherwise use the order balance."""
        amount = self.pending.amount
                
        # otherwise use the order balance.
        if amount == Decimal('0.00'):
            amount = self.order.balance
                    
        self.amount = amount

class ProcessorResult(object):
    """The result from a processor.process call"""

    def __init__(self, processor, success, message, payment=None):
        """Initialize with:

        processor - the key of the processor setting the result
        success - boolean
        message - a lazy string label, such as _('OK)
        payment - an OrderPayment or OrderAuthorization
        """
        self.success = success
        self.processor = processor
        self.message = message
        self.payment = payment

    def __unicode__(self):
        if self.success:
            yn = _('Success')
        else:
            yn = _('Failure')

        return u"ProcessorResult: %s [%s] %s" % (self.processor, yn, self.message)



# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gobject
import gtk
from kiwi.component import get_utility

from gazpacho import gapi
from gazpacho.annotator import draw_annotations
from gazpacho.commandmanager import command_manager
from gazpacho.cursor import Cursor
from gazpacho.interfaces import IGazpachoApp
from gazpacho.popup import PlaceholderPopup
from gazpacho.util import get_parent

placeholder_xpm = [
    # columns rows colors chars-per-pixel
    "8 8 2 1",
    "  c #bbbbbb",
    ". c #d6d6d6",
    # pixels
    " .  .   ",
    ".    .  ",
    "      ..",
    "      ..",
    ".    .  ",
    " .  .   ",
    "  ..    ",
    "  ..    ",
    ]

MIN_WIDTH = MIN_HEIGHT = 20

class Placeholder(gtk.Widget):
    def __init__(self):
        gtk.Widget.__init__(self)

        self._xpm_data = placeholder_xpm
        self._pixmap = None

        self.set_flags(self.flags() | gtk.CAN_FOCUS)

        from gazpacho.dndhandlers import PlaceholderDnDHandler
        dndhandler = PlaceholderDnDHandler()
        dndhandler.connect_drop_handlers(self)

        self.show()

    def do_realize(self):
        self.set_flags(self.flags() | gtk.REALIZED)

        events = (gtk.gdk.EXPOSURE_MASK |
                  gtk.gdk.BUTTON_PRESS_MASK |
                  gtk.gdk.POINTER_MOTION_MASK)

        self.window = gtk.gdk.Window(self.get_parent_window(),
                                     x=self.allocation.x,
                                     y=self.allocation.y,
                                     width=self.allocation.width,
                                     height=self.allocation.height,
                                     window_type=gtk.gdk.WINDOW_CHILD,
                                     wclass=gtk.gdk.INPUT_OUTPUT,
                                     visual=self.get_visual(),
                                     colormap=self.get_colormap(),
                                     event_mask=self.get_events() | events)

        self.window.set_user_data(self)
        self.style.attach(self.window)
        self.style.set_background(self.window, gtk.STATE_NORMAL)

        if not self._pixmap:
            t = gtk.gdk.pixmap_colormap_create_from_xpm_d(None,
                                                          self.get_colormap(),
                                                          None,
                                                          self._xpm_data)
            self._pixmap = t[0] # t[1] is the transparent color

        self.window.set_back_pixmap(self._pixmap, False)

    def get_project(self):
        """Get the project to which this placeholder belong."""
        project = None
        parent = get_parent(self)
        if parent:
            project = parent.project
        return project

    def do_size_allocate(self, allocation):
        self.allocation = allocation
        if self.flags() & gtk.REALIZED:
            self.window.move_resize(*allocation)

    def do_size_request(self, requisition):
        requisition.width = MIN_WIDTH
        requisition.height = MIN_HEIGHT

    def do_expose_event(self, event):
        light_gc = self.style.light_gc[gtk.STATE_NORMAL]
        dark_gc = self.style.dark_gc[gtk.STATE_NORMAL]
        w, h = event.window.get_size()

        # These lines make the Placeholder looks like a button
        event.window.draw_line(light_gc, 0, 0, w - 1, 0)
        event.window.draw_line(light_gc, 0, 0, 0, h - 1)
        event.window.draw_line(dark_gc, 0, h -1, w - 1, h - 1)
        event.window.draw_line(dark_gc, w - 1, 0, w - 1, h - 1)

        draw_annotations(self, event.window)

        return False

    def do_motion_notify_event(self, event):
        Cursor.set_for_widget_adaptor(event.window,
                                      get_utility(IGazpachoApp).add_class)

        return False

    def do_button_press_event(self, event):
        if not self.flags() & gtk.HAS_FOCUS:
            self.grab_focus()

        if event.button == 1 and event.type == gtk.gdk.BUTTON_PRESS:
            app = get_utility(IGazpachoApp)
            # A widget type is selected in the palette.
            # Add a new widget of that type
            if app.add_class:
                gapi.create_gadget(app.get_current_project(), app.add_class,
                                   self)

            # Shift clicking circles through the widget tree by
            # choosing the parent of the currently selected widget.
            elif event.state & gtk.gdk.SHIFT_MASK:
                parent = get_parent(self)
                parent.project.selection.circle(self)

            # Control clicking adds or removes the widget from the
            # selection
            elif event.state & gtk.gdk.CONTROL_MASK:
                parent = get_parent(self)
                parent.project.selection.toggle(self)

            # otherwise we should be able to select placeholder
            # for paste operations
            else:
                parent = get_parent(self)
                parent.project.selection.set(self)

        elif event.button == 3 and event.type == gtk.gdk.BUTTON_PRESS:
            popup = PlaceholderPopup(command_manager, self)
            popup.pop(event)

        return True

    def do_popup_menu(self):
        popup = PlaceholderPopup(command_manager, self)
        popup.pop(None)
        return True

    def is_deletable(self):
        """
        Check if it's possible to delete the placeholder. Currently it
        is only possible to delete a placeholder if it's parent is a
        gtk.Box and it is not the only child.

        @return: True if it's possible to delete the placeholder
        @rtype: bool
        """
        # XXX this code is still a bit hackish and we only support
        # deleting placeholders that are part of a Box. We should look
        # into making this more generic and maybe ask the parent
        # widget (or its adaptor) if it's ok to delete the
        # placeholder.
        parent = get_parent(self)
        if not parent:
            return False

        gtk_parent = parent.widget
        if not isinstance(gtk_parent, gtk.Box):
            return False

        return len(gtk_parent.get_children()) > 1

gobject.type_register(Placeholder)

if __name__ == '__main__':
    win = gtk.Window()
    win.connect('destroy', gtk.main_quit)

    p = Placeholder()
    win.add(p)
    win.show_all()

    gtk.main()



from django.db.models import ForeignKey
from django.contrib.auth.models import User

import threadlocals


class UserField(ForeignKey):
    """ UserField

    By defaults, foreign key to User; null=True, blank=True
    """

    def __init__(self, **kwargs):
        kwargs.setdefault('null', True)
        kwargs.setdefault('blank', True)
        ForeignKey.__init__(self, User, **kwargs)

      
class CreatorField(UserField):
    """ CreatorField

    By default, sets editable=False, default=threadlocals.get_current_user
    """

    def __init__(self, **kwargs):
        kwargs.setdefault('editable', False)
        kwargs.setdefault('default', threadlocals.get_current_user)
        UserField.__init__(self, **kwargs)


class EditorField(CreatorField):
    """ EditorField

    By default, sets editable=False, default=threadlocals.get_current_user

    Sets value to get_current_user() on each save of the model.
    """

    def __init__(self, **kwargs):
        super(CreatorField, self).__init__(**kwargs)

    def pre_save(self, model_instance, add):
        value = threadlocals.get_current_user()
        setattr(model_instance, self.name, value)
        if value:
            value = value.pk
        setattr(model_instance, self.attname, value)
        return value



# Copyright (C) 2005 Red Hat, Inc.
#               2005 Johan Dahlin
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import random

import gtk
from gtk import gdk

from gazpacho.util import get_parent

use_cairo = False
try:
    from cairo import gtk as cairogtk
    use_cairo = True
except ImportError:
    pass

SELECTION_NODE_SIZE = 7
BORDER_WIDTH = 3

class IAnnotate:
    def __init__(self, widget, window):
        pass

    def draw_notes(self, x, y, width, height):
        pass

    def draw_border(self, x, y, width, height, color):
        pass

class CairoAnnotator(object):
    def __init__(self, widget, window):
        self._cr = cairogtk.gdk_cairo_create(window)
        self._cr.set_line_width(1.0)

    def draw_nodes(self, x, y, width, height):
        cr = self._cr

        cr.set_source_rgba(0.40, 0.0, 0.0, 0.75) # translucent dark red

        if width > SELECTION_NODE_SIZE and height > SELECTION_NODE_SIZE:
            cr.rectangle(x + 1, y + 1,
                         SELECTION_NODE_SIZE - 1, SELECTION_NODE_SIZE - 1)
            cr.rectangle(x + 1, y + height -SELECTION_NODE_SIZE,
                         SELECTION_NODE_SIZE - 1, SELECTION_NODE_SIZE - 1)
            cr.rectangle(x + width - SELECTION_NODE_SIZE, y + 1,
                         SELECTION_NODE_SIZE - 1, SELECTION_NODE_SIZE - 1)
            cr.rectangle(x + width - SELECTION_NODE_SIZE,
                         y + height - SELECTION_NODE_SIZE,
                         SELECTION_NODE_SIZE - 1, SELECTION_NODE_SIZE -1)
            cr.fill()

        cr.rectangle(x + 0.5, y + 0.5, width - 1, height - 1)
        cr.stroke()

    def draw_border(self, x, y, width, height, color):
        cr = self._cr

        for i in range(0, BORDER_WIDTH):
            if (i <= BORDER_WIDTH / 2):
                alpha = (i + 1.) / (BORDER_WIDTH)
            else:
                alpha = (BORDER_WIDTH - i + 0.) / (BORDER_WIDTH)

            cr.set_source_rgba(color[0], color[1], color[2],
                               alpha)
            cr.rectangle(x - BORDER_WIDTH / 2 + i + 0.5,
                         y - BORDER_WIDTH / 2 + i + 0.5,
                         width + 2 * (BORDER_WIDTH / 2 - i) - 1,
                         height + 2 * (BORDER_WIDTH / 2 - i) - 1)
            cr.stroke()

class GdkAnnotator(object):
    def __init__(self, widget, window):
        self._widget = widget
        self._window = window
        self._gc = gdk.GC(window, line_width=BORDER_WIDTH)

    def draw_nodes(self, x, y, width, height):
        window = self._window
        gc = self._widget.style.black_gc

        if width > SELECTION_NODE_SIZE and height > SELECTION_NODE_SIZE:
            window.draw_rectangle(gc, True, x, y,
                                  SELECTION_NODE_SIZE, SELECTION_NODE_SIZE)
            window.draw_rectangle(gc, True, x, y + height -SELECTION_NODE_SIZE,
                                  SELECTION_NODE_SIZE, SELECTION_NODE_SIZE)
            window.draw_rectangle(gc, True, x + width - SELECTION_NODE_SIZE, y,
                                  SELECTION_NODE_SIZE, SELECTION_NODE_SIZE)
            window.draw_rectangle(gc, True, x + width - SELECTION_NODE_SIZE,
                                  y + height - SELECTION_NODE_SIZE,
                                  SELECTION_NODE_SIZE, SELECTION_NODE_SIZE)
        window.draw_rectangle(gc, False, x, y, width - 1, height - 1)

    def draw_border(self, x, y, width, height, color):
        gc = self._gc
        gc.set_rgb_fg_color(gdk.Color(int(color[0] * 65535.999),
                                      int(color[1] * 65535.999),
                                      int(color[2] * 65535.999)))

        self._window.draw_rectangle(gc, False,
                                    x - (BORDER_WIDTH / 2),
                                    y - (BORDER_WIDTH / 2),
                                    width, height)

if use_cairo:
    Annotator = CairoAnnotator
else:
    Annotator = GdkAnnotator

def _get_window_positioned_in(widget):
    """This returns the window that the given widget's allocation is
    relative to.
    Usually this is widget.get_parent_window(). But if the widget is a
    toplevel, we use its own window, as it doesn't have a parent."""
    if widget.get_parent():
        return widget.get_parent_window()
    else:
        return widget.window

def _calculate_window_offset(gdkwindow):
    """ This calculates the offset of the given window within its toplevel.
    It also returns the toplevel """
    x = y = 0
    window = gdkwindow
    while window:
        if window.get_window_type() != gtk.gdk.WINDOW_CHILD:
            break
        tmp_x, tmp_y = window.get_position()
        x += tmp_x
        y += tmp_y
        window = window.get_parent()

    return window, x, y

def _get_border_color(widget):
    if not isinstance(widget, (gtk.Box, gtk.Table)):
        return None

    if not (widget.parent):
        return None

    if isinstance(widget.parent, (gtk.Notebook, gtk.Window)):
        return None

    colors = [(1, 0, 0),
              (0, 1, 0),
              (0, 0, 1),
              (1, 1, 0),
              (0, 0.6, 0.6),
              (1, 0.5, 0)]

    index = widget.get_data("gazpacho-border-color")
    if index == None:
        index = random.randint(0, len(colors) - 1)
        widget.set_data("gazpacho-border-color", index)

    return colors[index]

def _draw_box_borders(project, widget, expose_win, annotator):
    if not project._app._show_structure:
        return

    if not isinstance(widget, gtk.Container):
        return

    if not widget.flags() & gtk.MAPPED:
        return

    border_win = _get_window_positioned_in(widget)
    _, border_x, border_y = _calculate_window_offset(border_win)
    _, expose_x, expose_y = _calculate_window_offset(expose_win)

    if isinstance(widget, gtk.Button):
        return

    color = _get_border_color(widget)
    if color:
        allocation = widget.allocation
        annotator.draw_border(border_x + allocation.x - expose_x,
                              border_y + allocation.y - expose_y,
                              allocation.width,
                              allocation.height,
                              color)

    children = widget.get_children()
    for child in children:
        _draw_box_borders(project, child, expose_win, annotator)

def _can_draw_nodes(sel_widget, expose_win):
    """ This returns True if it is OK to draw the selection nodes for the given
    selected widget inside the given window that has received an expose event.
    This is true if the expose window is a descendent of the window to which
    widget->allocation is relative. (The check for a descendent preserves
    clipping in a situation like a viewport) """

    sel_win = _get_window_positioned_in(sel_widget)
    window = expose_win
    while window:
        if window == sel_win:
            return True
        window = window.get_parent()

    return False

def _draw_nodes(project, expose_widget, expose_win, annotator):
    """ This is called to redraw any selection nodes that intersect the given
    exposed window.  It steps through all the selected widgets, converts the
    coordinates so they are relative to the exposed window, then calls
    _draw_nodes if appropriate """

    # Calculate the offset of the expose window within its toplevel
    expose_toplevel, expose_win_x, expose_win_y = \
                     _calculate_window_offset(expose_win)

    expose_win_w, expose_win_h = expose_win.get_size()

    # Step through all the selected widgets in the project
    for sel_widget in project.selection:
        sel_win = _get_window_positioned_in(sel_widget)

        if sel_win is None:
            continue

        # Calculate the offset of the selected widget's window within
        # its toplevel
        sel_toplevel, sel_x, sel_y = _calculate_window_offset(sel_win)

        # We only draw the nodes if the window that got the expose event is
        # in the same toplevel as the selected widget
        if (expose_toplevel == sel_toplevel and
            _can_draw_nodes(sel_widget, expose_win)):
            x = sel_x + sel_widget.allocation.x - expose_win_x
            y = sel_y + sel_widget.allocation.y - expose_win_y
            w = sel_widget.allocation.width
            h = sel_widget.allocation.height

            # Draw the selection nodes if they intersect the
            # expose window bounds
            if (x < expose_win_w and x + w >= 0 and
                y < expose_win_h and y + h >= 0):
                annotator.draw_nodes(x, y, w, h)

def draw_annotations(expose_widget, expose_win):
    """ This is called to redraw any gazpacho annotations that intersect
    the given exposed window. We only draw nodes on windows that are
    actually owned by the widget. This keeps us from repeatedly
    drawing nodes for the same window in the same expose event. """

    from gazpacho.gadget import Gadget

    expose_gadget = Gadget.from_widget(expose_widget)
    if not expose_gadget:
        expose_gadget = get_parent(expose_widget)
    if not expose_gadget:
        return False

    project = expose_gadget.project

    if not expose_win.is_viewable():
        return False

    # Find the corresponding widget and gadget
    if expose_widget != expose_win.get_user_data():
        return False

    annotator = Annotator(expose_widget, expose_win)

    _draw_box_borders(project, expose_widget.get_toplevel(),
                      expose_win, annotator)
    _draw_nodes(project, expose_widget, expose_win, annotator)



import sys
#import mainapp
from notmm.utils.wsgilib import HTTPRequest
from test_support import (
    #WSGIControllerTestCase, 
    TestClient, 
    unittest,
    make_app,
    settings,
    ResponseClass
    )

from model import MessageManager
from apps.blogengine.test_tutorial import connection

class ModelTestCase(unittest.TestCase):
    def setUp(self):
        

        self.db = MessageManager(connection).db_connection
        self.author = self.db.Author.findone(username="Etienne Robillard")
        self.msg = MessageManager(connection,
            messageid="12300", 
            author=self.author, 
            content="test")

    def tearDown(self):
    #    self.db.close()
        pass

    #def test_populate(self):
    #    model.setup() 
    #    models = model.get_models()
    #    #print models

    def test_save(self):
        if self.msg.db_connection.conn._is_open:
            print "Database connection is open"
            try:
                self.msg.save(commit=False)
            except Exception, e:
                print "rollback() called"
                print e
                self.msg.db_connection.backend.rollback()




#import backends
#import interfaces
from .interfaces import TemplateLoaderFactory
from .requestcontext import RequestContext
from .views import direct_to_template
from .views.utils import get_template_loader

TemplateLoaderFactory = TemplateLoaderFactory

__all__ = ['interfaces', 'backends', 'TemplateLoaderFactory', 'RequestContext']



from django.template import Node, NodeList
from django.template import TemplateSyntaxError
from django.template import Library
from product.models import Product
from livesettings import config_value

register = Library()

def recentlyviewed(recent, slug=""):
    """Build a list of recent products, skipping the current one if given."""
    if slug:
        recent = [r for r in recent if slug != r.slug]
    
    rmax = config_value('PRODUCT','RECENT_MAX')
    if len(recent) > rmax:
        recent = recent[:rmax]
    return {
        'recent_products' : recent,
    }
register.inclusion_tag('recentlist/_recently_viewed.html', takes_context=False)(recentlyviewed)


'''
This extension is DEPRECATED. Please use the orderinglist SQLAlchemy
extension instead.

For details:
http://www.sqlalchemy.org/docs/05/reference/ext/orderinglist.html

For an Elixir example:
http://elixir.ematia.de/trac/wiki/Recipes/UsingEntityForOrderedList
or
http://elixir.ematia.de/trac/browser/elixir/0.7.0/tests/test_o2m.py#L155



An ordered-list plugin for Elixir to help you make an entity be able to be
managed in a list-like way. Much inspiration comes from the Ruby on Rails
acts_as_list plugin, which is currently more full-featured than this plugin.

Once you flag an entity with an `acts_as_list()` statement, a column will be
added to the entity called `position` which will be an integer column that is
managed for you by the plugin.  You can pass an alternative column name to
the plugin using the `column_name` keyword argument.

In addition, your entity will get a series of new methods attached to it,
including:

+----------------------+------------------------------------------------------+
| Method Name          | Description                                          |
+======================+======================================================+
| ``move_lower``       | Move the item lower in the list                      |
+----------------------+------------------------------------------------------+
| ``move_higher``      | Move the item higher in the list                     |
+----------------------+------------------------------------------------------+
| ``move_to_bottom``   | Move the item to the bottom of the list              |
+----------------------+------------------------------------------------------+
| ``move_to_top``      | Move the item to the top of the list                 |
+----------------------+------------------------------------------------------+
| ``move_to``          | Move the item to a specific position in the list     |
+----------------------+------------------------------------------------------+


Sometimes, your entities that represent list items will be a part of different
lists. To implement this behavior, simply pass the `acts_as_list` statement a
callable that returns a "qualifier" SQLAlchemy expression. This expression will
be added to the generated WHERE clauses used by the plugin.

Example model usage:

.. sourcecode:: python

    from elixir import *
    from elixir.ext.list import acts_as_list

    class ToDo(Entity):
        subject = Field(String(128))
        owner = ManyToOne('Person')

        def qualify(self):
            return ToDo.owner_id == self.owner_id

        acts_as_list(qualifier=qualify)

    class Person(Entity):
        name = Field(String(64))
        todos = OneToMany('ToDo', order_by='position')


The above example can then be used to manage ordered todo lists for people.
Note that you must set the `order_by` property on the `Person.todo` relation in
order for the relation to respect the ordering. Here is an example of using
this model in practice:

.. sourcecode:: python

    p = Person.query.filter_by(name='Jonathan').one()
    p.todos.append(ToDo(subject='Three'))
    p.todos.append(ToDo(subject='Two'))
    p.todos.append(ToDo(subject='One'))
    session.commit(); session.clear()

    p = Person.query.filter_by(name='Jonathan').one()
    p.todos[0].move_to_bottom()
    p.todos[2].move_to_top()
    session.commit(); session.clear()

    p = Person.query.filter_by(name='Jonathan').one()
    assert p.todos[0].subject == 'One'
    assert p.todos[1].subject == 'Two'
    assert p.todos[2].subject == 'Three'


For more examples, refer to the unit tests for this plugin.
'''

from elixir.statements import Statement
from elixir.events import before_insert, before_delete
from sqlalchemy import Column, Integer, select, func, literal, and_
import warnings

__all__ = ['acts_as_list']
__doc_all__ = []


def get_entity_where(instance):
    clauses = []
    for column in instance.table.primary_key.columns:
        instance_value = getattr(instance, column.name)
        clauses.append(column == instance_value)
    return and_(*clauses)


class ListEntityBuilder(object):

    def __init__(self, entity, qualifier=None, column_name='position'):
        warnings.warn("The act_as_list extension is deprecated. Please use "
                      "SQLAlchemy's orderinglist extension instead",
                      DeprecationWarning, stacklevel=6)
        self.entity = entity
        self.qualifier_method = qualifier
        self.column_name = column_name

    def create_non_pk_cols(self):
        if self.entity._descriptor.autoload:
            for c in self.entity.table.c:
                if c.name == self.column_name:
                    self.position_column = c
            if not hasattr(self, 'position_column'):
                raise Exception(
                    "Could not find column '%s' in autoloaded table '%s', "
                    "needed by entity '%s'." % (self.column_name,
                        self.entity.table.name, self.entity.__name__))
        else:
            self.position_column = Column(self.column_name, Integer)
            self.entity._descriptor.add_column(self.position_column)

    def after_table(self):
        position_column = self.position_column
        position_column_name = self.column_name

        qualifier_method = self.qualifier_method
        if not qualifier_method:
            qualifier_method = lambda self: None

        def _init_position(self):
            s = select(
                [(func.max(position_column)+1).label('value')],
                qualifier_method(self)
            ).union(
                select([literal(1).label('value')])
            )
            a = s.alias()
            # we use a second func.max to get the maximum between 1 and the
            # real max position if any exist
            setattr(self, position_column_name, select([func.max(a.c.value)]))

            # Note that this method could be rewritten more simply like below,
            # but because this extension is going to be deprecated anyway,
            # I don't want to risk breaking something I don't want to maintain.
#            setattr(self, position_column_name, select(
#                [func.coalesce(func.max(position_column), 0) + 1],
#                qualifier_method(self)
#            ))
        _init_position = before_insert(_init_position)

        def _shift_items(self):
            self.table.update(
                and_(
                    position_column > getattr(self, position_column_name),
                    qualifier_method(self)
                ),
                values={
                    position_column : position_column - 1
                }
            ).execute()
        _shift_items = before_delete(_shift_items)

        def move_to_bottom(self):
            # move the items that were above this item up one
            self.table.update(
                and_(
                    position_column >= getattr(self, position_column_name),
                    qualifier_method(self)
                ),
                values = {
                    position_column : position_column - 1
                }
            ).execute()

            # move this item to the max position
            # MySQL does not support the correlated subquery, so we need to
            # execute the query (through scalar()). See ticket #34.
            self.table.update(
                get_entity_where(self),
                values={
                    position_column : select(
                        [func.max(position_column) + 1],
                        qualifier_method(self)
                    ).scalar()
                }
            ).execute()

        def move_to_top(self):
            self.move_to(1)

        def move_to(self, position):
            current_position = getattr(self, position_column_name)

            # determine which direction we're moving
            if position < current_position:
                where = and_(
                    position <= position_column,
                    position_column < current_position,
                    qualifier_method(self)
                )
                modifier = 1
            elif position > current_position:
                where = and_(
                    current_position < position_column,
                    position_column <= position,
                    qualifier_method(self)
                )
                modifier = -1

            # shift the items in between the current and new positions
            self.table.update(where, values = {
                position_column : position_column + modifier
            }).execute()

            # update this item's position to the desired position
            self.table.update(get_entity_where(self)) \
                      .execute(**{position_column_name: position})

        def move_lower(self):
            # replace for ex.: p.todos.insert(x + 1, p.todos.pop(x))
            self.move_to(getattr(self, position_column_name) + 1)

        def move_higher(self):
            self.move_to(getattr(self, position_column_name) - 1)


        # attach new methods to entity
        self.entity._init_position = _init_position
        self.entity._shift_items = _shift_items
        self.entity.move_lower = move_lower
        self.entity.move_higher = move_higher
        self.entity.move_to_bottom = move_to_bottom
        self.entity.move_to_top = move_to_top
        self.entity.move_to = move_to


acts_as_list = Statement(ListEntityBuilder)



# This is an auto-generated Django model module.
# You'll have to do the following manually to clean this up:
#   * Rearrange models' order
#   * Make sure each model has one field with primary_key=True
#   * Make sure each ForeignKey has `on_delete` set to the desired behavior.
#   * Remove `managed = False` lines if you wish to allow Django to create, modify, and delete the table
# Feel free to rename the models, but don't rename db_table values or field names.
from __future__ import unicode_literals

from django.db import models


class DjangoMigrations(models.Model):
    id = models.IntegerField(primary_key=True)  # AutoField?
    app = models.CharField(max_length=255)
    name = models.CharField(max_length=255)
    applied = models.DateTimeField()

    class Meta:
        managed = False
        db_table = 'django_migrations'


class SandboxSqlalchemyUser(models.Model):
    id = models.IntegerField(primary_key=True)  # AutoField?
    first_name = models.CharField(max_length=30)
    last_name = models.CharField(max_length=30)

    class Meta:
        #managed = False
        db_table = 'sandbox_sqlalchemy_user'



"""
Each shipping option uses the data in an Order object to calculate the shipping cost and return the value
"""
from decimal import Decimal
from django.utils.translation import ugettext, ugettext_lazy as _
#from livesettings import config_value
from shipping.modules.base import BaseShipper

class Shipper(BaseShipper):
    id = "NoShipping"

    def __str__(self):
        """
        This is mainly helpful for debugging purposes
        """
        return "No shipping"

    def description(self):
        """
        A basic description that will be displayed to the user when selecting their shipping options
        """
        return _("No Shipping")

    def cost(self):
        """
        Complex calculations can be done here as long as the return value is a dollar figure
        """
        return Decimal("0.00")

    def method(self):
        """
        Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        """
        return ugettext('No Shipping')

    def expectedDelivery(self):
        """
        Can be a plain string or complex calcuation returning an actual date
        """
        return _("immediately")

    def valid(self, order=None):
        """
        Can do complex validation about whether or not this option is valid.
        For example, may check to see if the recipient is in an allowed country
        or location.
        """
        return True




import tidylib

def clean_html_document(html, opts={}, keep_doc=True):
    """Clean a HTML document using the tidylib module
    
    Returns a 2-items tuple with cleaned html fragment at position 0
    and validation errors at position 1. May also accept custom tidy options
    passed as keyword arguments. 
    """
    html, errors = tidylib.tidy_document(html, opts, keep_doc=keep_doc)
    return (html, errors) 



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

try:
    #Py3
    from urllib.parse import unquote_plus
except ImportError:
    from urllib import unquote_plus

from werkzeug.wrappers import BaseRequest
from werkzeug.formparser import parse_form_data
from werkzeug import ImmutableMultiDict

__all__ = ['HTTPRequest']

class HTTPRequest(BaseRequest):
    """A generic HTTP request object."""
    
    def __init__(self, environ, **kwargs):
        """provide a generic environment for HTTP requests"""
        super(HTTPRequest, self).__init__(environ, **kwargs)
        
        self.query_args =  self.args

    def get_remote_user(self):
        '''Subclasses should override this method to retrieve a User storage class
        programmatically.'''
        # Adds a copy of the user settings to the
        # session store
        user = self.environ.get('REMOTE_USER', None)
        return user

    @property
    def user(self):
        """Returns the current user as defined in environ['REMOTE_USER'] or
        None if not set"""
        return self.get_remote_user()

    def get_user(self):
        return self.user

    def get_full_path(self):
        """Return the value of PATH_INFO, a web browser dependent
        HTTP header, or None if the value is not set"""

        try:
            p = unquote_plus(self.environ['PATH_INFO'])
        except KeyError:
            # invalid CGI environment
            return None
        return p    
            
    def get_POST(self, keep_blank_values=True, strict_parsing=False):
        """Extracts data from a POST request
        Returns a dict instance with extracted keys/values pairs."""
        if not (self.method == 'POST' or 'wsgi.input' in self.environ):
            return {}
        fs_environ = self.environ.copy()
        stream, form, files = parse_form_data(fs_environ)
        return ImmutableMultiDict(form)

    # extra public methods borrowed from Django
    def is_ajax(self):
        """check if the http request was transmitted with asyncronous (AJAX) transport"""
        if 'HTTP_X_REQUESTED_WITH' in self.environ:
            if self.environ['HTTP_X_REQUESTED_WITH'] is 'XMLHttpRequest':
                return True
        #print 'not ajax'        
        return False        

    
    def is_secure(self):
        return bool(self.environ.get("HTTPS") == "on")
    
    @property
    def method(self):
        return str(self.environ.get('REQUEST_METHOD'))
    
    @property
    def POST(self):
        return self.get_POST()
    
    @property
    def GET(self):
        return getattr(self, 'query_args', {})

    path_url = property(get_full_path)



from datetime import datetime
from decimal import Decimal
from django.test import TestCase
from models import Carrier, ProductShippingPrice, Shipper
from product.models import Product

class ProductShippingSimpleTest(TestCase):
    fixtures = ['products.yaml',]

    def testCreate(self):
        c = Carrier(key="test", active=True)
        c.save()
        product = Product.objects.get(slug='dj-rocks')
        p = ProductShippingPrice(carrier=c, 
            product=product,
            price=Decimal("10.00"),
            )
        p.save()
        
        self.assertEqual(c.price(product), Decimal("10.00"))
        
        
class ProductShippingPricingTest(TestCase):
    fixtures = ['products.yaml']

    def setUp(self):
        self.carrier = Carrier(name="pricing", active=True)
        self.carrier.save()
        self.product = Product.objects.get(slug='dj-rocks')
        t = ProductShippingPrice(carrier=self.carrier,
            product=self.product,
            price=Decimal("10.00"),
            )
        t.save()
        
    def testBase(self):
        self.assertEqual(self.carrier.price(self.product), Decimal("10.00"))
        
    def test2Prices(self):
        c2 = Carrier(name="test2", active=True)
        c2.save()
        t = ProductShippingPrice(carrier=c2,
            product=self.product,
            price=Decimal("20.00"),
            )
        t.save()
        
        self.assertEqual(self.carrier.price(self.product), Decimal("10.00"))
        self.assertEqual(c2.price(self.product), Decimal("20.00"))

    def testVariantPrices(self):
        """price for shipping a variant is the same as the master, except when it is explicitly set"""
        
        variant = Product.objects.get(slug='dj-rocks-s-b')
        
        self.assertEqual(self.carrier.price(variant), Decimal("10.00"))
        
        t = ProductShippingPrice(carrier=self.carrier,
            product=variant,
            price=Decimal("20.00"),
            )
        t.save()
        
        self.assertEqual(self.carrier.price(variant), Decimal("20.00"))
        



from django import forms
from django.db.models.query import EmptyQuerySet 
from shipping.modules.tieredweight.models import Carrier, Zone


class CarrierAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(CarrierAdminForm, self).__init__(*args, **kwargs)
        if 'instance' in kwargs:
            tmp_qs = self.fields['default_zone'].queryset
            self.fields['default_zone'].queryset = tmp_qs.filter(carrier=kwargs['instance'])
        else:
            self.fields['default_zone'].queryset = EmptyQuerySet()

    class Meta:
        model = Carrier


class ZoneAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(ZoneAdminForm, self).__init__(*args, **kwargs)
        zones = None
        if 'instance' in kwargs:
            carrier = kwargs['instance'].carrier.pk
            zones = Carrier.objects.get(pk=carrier).zones.exclude(pk=kwargs['instance'].pk)
        else:
            try:
                carrier = args[0]['carrier']
                if carrier:
                    zones = Carrier.objects.get(pk=carrier).zones.all()
            except IndexError:
                pass

        if zones:
            used_countries = []
            for zone in zones:
                for country in zone.countries.all():
                    used_countries.append(country.name)
            tmp_qs = self.fields['countries'].queryset
            self.fields['countries'].queryset = tmp_qs.exclude(name__in=used_countries)

    class Meta:
        model = Zone


from decimal import Decimal
from django import http
from django.core.xheaders import populate_xheaders
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.template.loader import select_template
from django.utils.translation import ugettext as _
from l10n.utils import moneyfmt
from livesettings.functions import config_value
from product import signals
from product.models import Category, Product, ConfigurableProduct, sorted_tuple
from product.signals import index_prerender
from product.utils import find_best_auto_discount
from satchmo_utils.json import json_encode
from satchmo_utils.numbers import  RoundedDecimalError, round_decimal
from satchmo_utils.views import bad_or_missing
import datetime
import logging
import random

log = logging.getLogger('product.views')

NOTSET = object()

# ---- Helpers ----

def find_product_template(product, producttypes=None):
    """Searches for the correct override template given a product."""
    if producttypes is None:
        producttypes = product.get_subtypes()

    templates = ["product/detail_%s.html" % x.lower() for x in producttypes]
    templates.append('product/product.html')
    log.debug("finding product template: %s", templates)
    return select_template(templates)
    
def optionids_from_post(configurableproduct, POST):
    """Reads through the POST dictionary and tries to match keys to possible `OptionGroup` ids
    from the passed `ConfigurableProduct`"""
    chosen_options = []
    for opt_grp in configurableproduct.option_group.all():
        if POST.has_key(str(opt_grp.id)):
            chosen_options.append('%s-%s' % (opt_grp.id, POST[str(opt_grp.id)]))
    return sorted_tuple(chosen_options)

# ---- Views ----
def category_index(request, template="product/category_index.html", root_only=True):
    """Display all categories.
    
    Parameters:
    - root_only: If true, then only show root categories.
    """
    cats = Category.objects.root_categories()
    ctx = {
        'categorylist' : cats,
    }
    return render_to_response(template, context_instance=RequestContext(request, ctx))

def category_view(request, slug, parent_slugs='', template='product/category.html'):
    """Display the category, its child categories, and its products.

    Parameters:
     - slug: slug of category
     - parent_slugs: ignored    
    """
    try:
        category =  Category.objects.get_by_site(slug=slug)
        products = list(category.active_products())
        sale = find_best_auto_discount(products)

    except Category.DoesNotExist:
        return bad_or_missing(request, _('The category you have requested does not exist.'))

    child_categories = category.get_all_children()

    ctx = {
        'category': category, 
        'child_categories': child_categories,
        'sale' : sale,
        'products' : products,
    }
    index_prerender.send(Product, request=request, context=ctx, category=category, object_list=products)
    return render_to_response(template, context_instance=RequestContext(request, ctx))


def display_featured(num_to_display=NOTSET, random_display=NOTSET):
    """
    Used by the index generic view to choose how the featured products are displayed.
    Items can be displayed randomly or all in order
    """
    if num_to_display == NOTSET:
        num_to_display = config_value('PRODUCT','NUM_DISPLAY')
    if random_display == NOTSET:
        random_display = config_value('PRODUCT','RANDOM_FEATURED')
        
    q = Product.objects.featured_by_site()
    if not random_display:
        return q[:num_to_display]
    else:
        return q.order_by('?')[:num_to_display]

def get_configurable_product_options(request, id):
    """Used by admin views"""
    cp = get_object_or_404(ConfigurableProduct, product__id=id)
    options = ''
    for og in cp.option_group.all():
        for opt in og.option_set.all():
            options += '<option value="%s">%s</option>' % (opt.id, str(opt))
    if not options:
        return '<option>No valid options found in "%s"</option>' % cp.product.slug
    return http.HttpResponse(options, mimetype="text/html")


def get_product(request, product_slug=None, selected_options=(), 
    default_view_tax=NOTSET):
    """Basic product view"""
    
    errors = request.session.get('ERRORS', None)
    if errors is not None:
        del(request.session['ERRORS'])
    try:
        product = Product.objects.get_by_site(active=True, slug=product_slug)
    except Product.DoesNotExist:
        return bad_or_missing(request, _('The product you have requested does not exist.'))

    if default_view_tax == NOTSET:
        default_view_tax = config_value('TAX', 'DEFAULT_VIEW_TAX')

    subtype_names = product.get_subtypes()
    
    # Save product id for xheaders, in case we display a ConfigurableProduct
    product_id = product.id
    
    # Clone product object in order to have current product variations in context (extra_context)
    current_product = product

    if 'ProductVariation' in subtype_names:
        selected_options = product.productvariation.unique_option_ids
        #Display the ConfigurableProduct that this ProductVariation belongs to.
        product = product.productvariation.parent.product
        subtype_names = product.get_subtypes()

    best_discount = find_best_auto_discount(product)
    
    extra_context = {
        'product': product,
        'current_product' : current_product,
        'default_view_tax': default_view_tax,
        'sale': best_discount,
        'error_message' : errors,
    }

    # Get the template context from the Product.
    extra_context = product.add_template_context(context=extra_context,
        request=request, selected_options=selected_options,
        default_view_tax=default_view_tax)

    template = find_product_template(product, producttypes=subtype_names)
    context = RequestContext(request, extra_context)
    
    response = http.HttpResponse(template.render(context))
    populate_xheaders(request, response, Product, product_id)
    return response

def get_price(request, product_slug):
    """Get base price for a product, returning the answer encoded as JSON."""
    quantity = Decimal('1')

    try:
        product = Product.objects.get_by_site(active=True, slug=product_slug)
    except Product.DoesNotExist:
        return http.HttpResponseNotFound(json_encode(('', _("not available"))), mimetype="text/javascript")

    prod_slug = product.slug

    if request.method == "POST" and request.POST.has_key('quantity'):
        try:
            quantity = round_decimal(request.POST['quantity'], places=2, roundfactor=.25)
        except RoundedDecimalError:
            quantity = Decimal('1.0')
            log.warn("Could not parse a decimal from '%s', returning '1.0'", request.POST['quantity'])

    if 'ConfigurableProduct' in product.get_subtypes():
        cp = product.configurableproduct
        chosen_options = optionids_from_post(cp, request.POST)
        pvp = cp.get_product_from_options(chosen_options)

        if not pvp:
            return http.HttpResponse(json_encode(('', _("not available"))), mimetype="text/javascript")
        prod_slug = pvp.slug
        price = moneyfmt(pvp.get_qty_price(quantity))
    else:
        price = moneyfmt(product.get_qty_price(quantity))

    if not price:
        return http.HttpResponse(json_encode(('', _("not available"))), mimetype="text/javascript")

    return http.HttpResponse(json_encode((prod_slug, price)), mimetype="text/javascript")


def get_price_detail(request, product_slug):
    """Get all price details for a product, returning the response encoded as JSON."""
    results = {
        "success" : False,
        "message" :  _("not available")
    }
    price = None

    if request.method=="POST":
        reqdata = request.POST
    else:
        reqdata = request.GET

    try:
        product = Product.objects.get_by_site(active=True, slug=product_slug)
        found = True

        prod_slug = product.slug

        if reqdata.has_key('quantity'):
            try:
                quantity = round_decimal(reqdata['quantity'], places=2, roundfactor=.25)
            except RoundedDecimalError:
                quantity = Decimal('1.0')
                log.warn("Could not parse a decimal from '%s', returning '1.0'", reqdata['quantity'])
        else:
            quantity = Decimal('1.0')

        if 'ConfigurableProduct' in product.get_subtypes():
            cp = product.configurableproduct
            chosen_options = optionids_from_post(cp, reqdata)
            product = cp.get_product_from_options(chosen_options)

        if product:
            price = product.get_qty_price(quantity)

            results['slug'] = product.slug
            results['price'] = float(price)
            results['success'] = True
            results['message'] = ""

    except Product.DoesNotExist:
        found = False

    data = json_encode(results)
    if found:
        return http.HttpResponse(data, mimetype="text/javascript")
    else:
        return http.HttpResponseNotFound(data, mimetype="text/javascript")



from decimal import Decimal    
from django.test import TestCase
from satchmo_utils.numbers import round_decimal, RoundedDecimalError, trunc_decimal

class TestRoundedDecimals(TestCase):
    
    def testRoundingDecimals(self):
        """Test Partial Unit Rounding Decimal Conversion behavior"""
        val = round_decimal(val=3.40, places=5, roundfactor=.5, normalize=True)
        self.assertEqual(val, Decimal("3.5"))
        
        val = round_decimal(val=3.40, places=5, roundfactor=-.5, normalize=True)
        self.assertEqual(val, Decimal("3"))
        
        val = round_decimal(val=0, places=5, roundfactor=-.5, normalize=False)
        self.assertEqual(val, Decimal("0.00000"))
        
        val = round_decimal(0, 5, -.5, False)
        self.assertEqual(val, Decimal("0.00000"))
        
        val = round_decimal(0)
        self.assertEqual(val, Decimal("0"))
        
        val = round_decimal(3.23,4,-.25)
        self.assertEqual(val, Decimal("3"))
        
        val = round_decimal(-3.23,4,-.25)
        self.assertEqual(val, Decimal("-3"))
        val = round_decimal(-3.23,4,.25)
        self.assertEqual(val, Decimal("-3.25"))
        
        val = round_decimal(3.23,4,.25)
        self.assertEqual(val, Decimal("3.25"))
        
        val = round_decimal(3.23,4,.25,False)
        self.assertEqual(val, Decimal("3.2500"))
        
        val = round_decimal(3.23,1,.25,False)
        self.assertEqual(val, Decimal("3.2"))
        
        val = round_decimal(2E+1, places=2)
        self.assertEqual(val, Decimal('20.00'))

    def testTruncDecimal(self):
        """Test trunc_decimal's rounding behavior."""
        # val = trunc_decimal("0.004", 2)
        # self.assertEqual(val, Decimal("0.00"))
        val = trunc_decimal("0.005", 2)
        self.assertEqual(val, Decimal("0.01"))
        val = trunc_decimal("0.009", 2)
        self.assertEqual(val, Decimal("0.01"))

        val = trunc_decimal("2E+1", places=2)
        self.assertEqual(val, Decimal('20.00'))
        
        val = trunc_decimal(2.1E+1, places=2)
        self.assertEqual(val, Decimal('21.00'))

        val = trunc_decimal(2.1223E+1, places=2)
        self.assertEqual(val, Decimal('21.23'))

        val = trunc_decimal("2.1223E+1", places=2)
        self.assertEqual(val, Decimal('21.23'))



import shipper
from livesettings import config_choice_values

def get_methods():
  '''
    Fires off shipper.Shipper() for each choice that's
    enabled in /settings/
  '''

  return [shipper.Shipper(service_type=value) for value in config_choice_values('shipping.modules.canadapost', 'CANADAPOST_SHIPPING_CHOICES')]



from test_support import unittest
import sys
import os
import time
import urllib
import logging

from notmm.utils.configparse import string_getter, int_getter

try:
    import termcolor
    _colored = termcolor.colored
except (ImportError, AttributeError):
    from django.utils.termcolors import colorize as _colored


class TermEventManager(object):

    fp = sys.stdout

    def _stdout(self, s):
        self.fp.write(str(s + os.linesep))

    def _colored(self, msg, **kwargs):
        """Write a string to the terminal with colored output"""
        _stdout(_colored(msg, **kwargs))

    def write(self, s, event_class='info'):
        emitter = self.system_event_types[event_class]
        emitter(s)

    system_event_types = {
        'info': lambda s: _colored(s, color='blue', attrs=['bold']),
        'debug': lambda s: _colored(s, color='white', attrs=['bold']),
        'warning': lambda s: _colored(s, color='orange', attrs=['bold'])
    }

RootEvent = TermEventManager()
RootEvent.write('system init')

class EventManagerTestCase(unittest.TestCase):
    def test_base(self):
        RootEvent.write('beauty is an artificial life', event_class="info")





"""
Stores customer, organization, and order information.
"""
from django.conf import settings
from django.contrib.auth.models import User
from django.core import urlresolvers
from django.db import models
from django.utils.translation import ugettext, ugettext_lazy as _
from livesettings import config_get_group
from l10n.models import Country
from satchmo_store.contact import CUSTOMER_ID
import datetime
import logging
import sys

log = logging.getLogger('contact.models')

class ContactRole(models.Model):
    key = models.CharField(_('Key'), max_length=30, unique=True, primary_key=True)
    name = models.CharField(_('Name'), max_length=40)

    def __unicode__(self):
        return ugettext(self.name)


class ContactOrganization(models.Model):
    key = models.CharField(_('Key'), max_length=30, unique=True, primary_key=True)
    name = models.CharField(_('Name'), max_length=40)

    def __unicode__(self):
        return ugettext(self.name)
        
    class Meta:
        verbose_name = _('Contact organization type')


class ContactOrganizationRole(models.Model):
    key = models.CharField(_('Key'), max_length=30, unique=True, primary_key=True)
    name = models.CharField(_('Name'), max_length=40)

    def __unicode__(self):
        return ugettext(self.name)

class ContactInteractionType(models.Model):
    key = models.CharField(_('Key'), max_length=30, unique=True, primary_key=True)
    name = models.CharField(_('Name'), max_length=40)

    def __unicode__(self):
        return ugettext(self.name)


class OrganizationManager(models.Manager):
    def by_name(self, name, create=False, role='Customer', orgtype='Company'):        
        org = None
        orgs = self.filter(name=name, role__key=role, type__key=orgtype)
        if orgs.count() > 0:
            org = orgs[0]
            
        if not org:
            if not create:
                raise Organization.DoesNotExist()
            else:
                log.debug('Creating organization: %s', name)
                role = ContactOrganizationRole.objects.get(pk=role)
                orgtype = ContactOrganization.objects.get(pk=orgtype)
                org = Organization(name=name, role=role, type=orgtype)
                org.save()
        
        return org

class Organization(models.Model):
    """
    An organization can be a company, government or any kind of group.
    """
    name = models.CharField(_("Name"), max_length=50, )
    type = models.ForeignKey(ContactOrganization, verbose_name=_("Type"), null=True)
    role = models.ForeignKey(ContactOrganizationRole, verbose_name=_("Role"), null=True)    
    create_date = models.DateField(_("Creation Date"))
    notes = models.TextField(_("Notes"), max_length=200, blank=True, null=True)

    objects = OrganizationManager()
    
    def __unicode__(self):
        return self.name

    def save(self, **kwargs):
        """Ensure we have a create_date before saving the first time."""
        if not self.pk:
            self.create_date = datetime.date.today()
        super(Organization, self).save(**kwargs)

    class Meta:
        verbose_name = _("Organization")
        verbose_name_plural = _("Organizations")

class ContactManager(models.Manager):

    def from_request(self, request, create=False):
        """Get the contact from the session, else look up using the logged-in
        user. Create an unsaved new contact if `create` is true.

        Returns:
        - Contact object or None
        """
        contact = None
        if request.session.get(CUSTOMER_ID):
            try:
                contact = Contact.objects.get(id=request.session[CUSTOMER_ID])
            except Contact.DoesNotExist:
                del request.session[CUSTOMER_ID]
            
        if contact is None and request.user.is_authenticated():
            try:
                contact = Contact.objects.get(user=request.user.id)
                request.session[CUSTOMER_ID] = contact.id
            except Contact.DoesNotExist:
                pass
        else:
            # Don't create a Contact if the user isn't authenticated.
            create = False

        if contact is None:
            if create:
                contact = Contact(user=request.user)

            else:
                raise Contact.DoesNotExist()

        return contact


class Contact(models.Model):
    """
    A customer, supplier or any individual that a store owner might interact
    with.
    """
    title = models.CharField(_("Title"), max_length=30, blank=True, null=True)
    first_name = models.CharField(_("First name"), max_length=30, )
    last_name = models.CharField(_("Last name"), max_length=30, )
    user = models.ForeignKey(User, blank=True, null=True, unique=True)
    role = models.ForeignKey(ContactRole, verbose_name=_("Role"), null=True)    
    organization = models.ForeignKey(Organization, verbose_name=_("Organization"), blank=True, null=True)
    dob = models.DateField(_("Date of birth"), blank=True, null=True)
    email = models.EmailField(_("Email"), blank=True, max_length=75)
    notes = models.TextField(_("Notes"), max_length=500, blank=True)
    create_date = models.DateField(_("Creation date"))

    objects = ContactManager()

    def _get_full_name(self):
        """Return the person's full name."""
        return u'%s %s' % (self.first_name, self.last_name)
    full_name = property(_get_full_name)

    def _shipping_address(self):
        """Return the default shipping address or None."""
        try:
            return self.addressbook_set.get(is_default_shipping=True)
        except AddressBook.DoesNotExist:
            return None
    shipping_address = property(_shipping_address)

    def _billing_address(self):
        """Return the default billing address or None."""
        try:
            return self.addressbook_set.get(is_default_billing=True)
        except AddressBook.DoesNotExist:
            return None
    billing_address = property(_billing_address)

    def _primary_phone(self):
        """Return the default phone number or None."""
        try:
            return self.phonenumber_set.get(primary=True)
        except PhoneNumber.DoesNotExist:
            return None
    primary_phone = property(_primary_phone)

    def __unicode__(self):
        return self.full_name

    def save(self, **kwargs):
        """Ensure we have a create_date before saving the first time."""
        if not self.pk:
            self.create_date = datetime.date.today()
        # Validate contact to user sync
        if self.user:
            dirty = False
            user = self.user
            if user.email != self.email:
                user.email = self.email
                dirty = True
            
            if user.first_name != self.first_name:
                user.first_name = self.first_name
                dirty = True

            if user.last_name != self.last_name:
                user.last_name = self.last_name
                dirty = True
                
            if dirty:
                self.user = user
                self.user.save()

        super(Contact, self).save(**kwargs)

    class Meta:
        verbose_name = _("Contact")
        verbose_name_plural = _("Contacts")        

PHONE_CHOICES = (
    ('Work', _('Work')),
    ('Home', _('Home')),
    ('Fax', _('Fax')),
    ('Mobile', _('Mobile')),
)

class Interaction(models.Model):
    """
    A type of activity with the customer.  Useful to track emails, phone calls,
    or in-person interactions.
    """
    contact = models.ForeignKey(Contact, verbose_name=_("Contact"))
    type = models.ForeignKey(ContactInteractionType, verbose_name=_("Type"))
    date_time = models.DateTimeField(_("Date and Time"), )
    description = models.TextField(_("Description"), max_length=200)

    def __unicode__(self):
        return u'%s - %s' % (self.contact.full_name, self.type)

    class Meta:
        verbose_name = _("Interaction")
        verbose_name_plural = _("Interactions")

class PhoneNumber(models.Model):
    """
    Phone number associated with a contact.
    """
    contact = models.ForeignKey(Contact)
    type = models.CharField(_("Description"), choices=PHONE_CHOICES,
        max_length=20, blank=True)
    phone = models.CharField(_("Phone Number"), blank=True, max_length=30,
        )
    primary = models.BooleanField(_("Primary"), default=False)

    def __unicode__(self):
        return u'%s - %s' % (self.type, self.phone)

    def save(self, **kwargs):
        """
        If this number is the default, then make sure that it is the only
        primary phone number. If there is no existing default, then make
        this number the default.
        """
        existing_number = self.contact.primary_phone
        if existing_number:
            if self.primary:
                existing_number.primary = False
                super(PhoneNumber, existing_number).save()
        else:
            self.primary = True
        super(PhoneNumber, self).save(**kwargs)

    class Meta:
        ordering = ['-primary']
        verbose_name = _("Phone Number")
        verbose_name_plural = _("Phone Numbers")

class AddressBook(models.Model):
    """
    Address information associated with a contact.
    """
    contact = models.ForeignKey(Contact)
    description = models.CharField(_("Description"), max_length=20, blank=True,
        help_text=_('Description of address - Home, Office, Warehouse, etc.',))
    addressee = models.CharField(_("Addressee"), max_length=80)
    street1 = models.CharField(_("Street"), max_length=80)
    street2 = models.CharField(_("Street"), max_length=80, blank=True)
    state = models.CharField(_("State"), max_length=50, blank=True)
    city = models.CharField(_("City"), max_length=50)
    postal_code = models.CharField(_("Zip Code"), max_length=30)
    country = models.ForeignKey(Country, verbose_name=_("Country"))
    is_default_shipping = models.BooleanField(_("Default Shipping Address"),
        default=False)
    is_default_billing = models.BooleanField(_("Default Billing Address"),
        default=False)

    def __unicode__(self):
       return u'%s - %s' % (self.contact.full_name, self.description)

    def save(self, **kwargs):
        """
        If this address is the default billing or shipping address, then
        remove the old address's default status. If there is no existing
        default, then make this address the default.
        """
        existing_billing = self.contact.billing_address
        if existing_billing:
            if self.is_default_billing:
                existing_billing.is_default_billing = False
                super(AddressBook, existing_billing).save()
        else:
            self.is_default_billing = True

        existing_shipping = self.contact.shipping_address
        if existing_shipping:
            if self.is_default_shipping:
                existing_shipping.is_default_shipping = False
                super(AddressBook, existing_shipping).save()
        else:
            self.is_default_shipping = True

        super(AddressBook, self).save(**kwargs)

    class Meta:
        verbose_name = _("Address Book")
        verbose_name_plural = _("Address Books")

import config



from django.conf import settings
from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.core.files import File
from django.core.files.storage import default_storage
from django.http import HttpResponse
from django.test import TestCase
from django.test.client import Client
from l10n.models import Country
from livesettings import config_value
from product.models import Product
from product.modules.downloadable.models import DownloadLink, DownloadableProduct
from satchmo_store.contact.models import AddressBook, Contact
from satchmo_store.shop.models import Cart, Order
from satchmo_store.shop.signals import sendfile_url_for_file
from shipping.modules.flat.shipper import Shipper as flat
from shipping.modules.per.shipper import Shipper as per
from satchmo_store.shop import get_satchmo_setting

import datetime
from decimal import Decimal
import keyedcache
import os
from shutil import rmtree
from tempfile import mkdtemp, mkstemp

prefix = get_satchmo_setting('SHOP_BASE')
if prefix == '/':
    prefix = ''

class DownloadableShippingTest(TestCase):

    fixtures = ['l10n-data.yaml','test_shop.yaml']

    def setUp(self):
        self.site = Site.objects.get_current()
        self.product1 = Product.objects.create(slug='p1', name='p1', site=self.site)
        self.cart1 = Cart.objects.create(site=self.site)
        self.cartitem1 = self.cart1.add_item(self.product1, 3)

    def tearDown(self):
        keyedcache.cache_delete()

    def test_downloadable_zero_shipping(self):
        subtype2 = DownloadableProduct.objects.create(product=self.product1)
        self.assertEqual(self.product1.get_subtypes(), ('DownloadableProduct',))

        self.assertFalse(subtype2.is_shippable)
        self.assertFalse(self.product1.is_shippable)
        self.assertFalse(self.cart1.is_shippable)
        self.assertEqual(flat(self.cart1, None).cost(), Decimal("0.00"))
        self.assertEqual(per(self.cart1, None).cost(), Decimal("0.00"))

class DownloadableProductTest(TestCase):
    fixtures = ['l10n-data.yaml', 'products.yaml']

    def setUp(self):
        self.site = Site.objects.get_current()

        #
        # setup the protected dir; since we're using the default storage class,
        # this will point to
        #
        #   /path/to/static/protected/
        #
        # where "/path/to/static/" is your settings.MEDIA_ROOT and "protected"
        # is your PRODUCT.PROTECTED_DIR setting.
        #
        self.protected_dir = default_storage.path(
            config_value('PRODUCT', 'PROTECTED_DIR')
        )
        if not os.path.exists(self.protected_dir):
            os.makedirs(self.protected_dir)

        # setup a temporary file in the protected dir: this is the file that
        # django will use during this test, but we won't use it; close and
        # remove it.
        _file, _abs_path = mkstemp(dir=self.protected_dir)
        os.close(_file)
        os.remove(_abs_path)
        self.file_name = os.path.basename(_abs_path)

        # setup a temporary source dir and source file, using the same file name
        # generated eariler.
        self.dir = mkdtemp()
        self.file = open(os.path.join(self.dir, self.file_name), "w")

        # a fake SHA
        self.key = "".join(["12abf" for i in range(8)])

        # setup a contact
        c, _created = Contact.objects.get_or_create(
            first_name="Jim",
            last_name="Tester",
            email="Jim@JimWorld.com",
        )
        ad, _created = AddressBook.objects.get_or_create(
            contact=c, description="home",
            street1 = "test", state="OR", city="Portland",
            country = Country.objects.get(iso2_code__iexact = 'US'),
            is_default_shipping=True,
            is_default_billing=True,
        )

        # setup a order
        o, _created = Order.objects.get_or_create(
            contact=c, shipping_cost=Decimal('6.00'), site=self.site
        )

        # setup download
        self.product, _created = DownloadableProduct.objects.get_or_create(
            product=Product.objects.get(slug='dj-rocks'),
            file=File(self.file),
            num_allowed_downloads=3,
            expire_minutes=1,
        )
        self.product_link, _created = DownloadLink.objects.get_or_create(
            downloadable_product=self.product,
            order=o, key=self.key, num_attempts=0,
            time_stamp=datetime.datetime.now()
        )

        # setup client
        self.domain = 'satchmoserver'
        self.client = Client(SERVER_NAME=self.domain)

        # go through the verification step
        self.pd_url = urlresolvers.reverse(
            'satchmo_download_send', kwargs= {'download_key': self.key}
        )
        pd_process_url = urlresolvers.reverse(
            'satchmo_download_process', kwargs= {'download_key': self.key}
        )

        # first, hit the url.
        response = self.client.get(self.pd_url)
        self.assertEqual(response['Location'],
            'http://%s%s' % (self.domain, pd_process_url)
        )

        # follow the redirect to "process" the key.
        response = self.client.get(response['Location'])
        self.assertEqual(self.client.session.get('download_key', None), self.key)

    def tearDown(self):
        self.file.close()
        rmtree(self.dir)
        os.remove(os.path.join(self.protected_dir, self.file_name))

    def test_download_urls(self):
        """
        Test that download urls remain unchanged after changeset
        hg:4d23ed40f534/git:d06b4ec
        """
        check_url = "%s/download/send/%s/" % (prefix, self.key)
        self.assertEqual(self.pd_url, check_url)

    def test_download_link(self):
        """Test that we are able to download a product."""

        # we should have gotten a page that says "click here".
        # hit the pd_url again; we should get a sendfile redirect.
        response = self.client.get(self.pd_url)

        exp_file_name = "attachment; filename=%s" % self.file_name
        self.assertEqual(response['Content-Disposition'], exp_file_name)

        #
        # we expect something like:
        #
        #   /protected/file_name
        #
        exp_url = "/%s/%s" % (
            self.protected_dir.split(os.path.sep)[-1],
            self.file_name
        )

        self.assertEqual(response['X-Accel-Redirect'], exp_url)
        self.assertEqual(response['X-Sendfile'], exp_url)
        self.assertEqual(response['X-LIGHTTPD-send-file'], exp_url)

    def test_sendfile_signal(self):
        """Test that we can modify the location in the sendfile header."""

        # something else instead of the usual /protected/file_name
        str_format = '/sendfile_dir/%s'
        exp_url = str_format % self.file_name

        # hookup a listener.
        def _sendfile_listener(sender, file=None, product=None, url_dict={},
            **kwargs):
            file_name = os.path.basename(file.name)
            url_dict['url'] = str_format % file_name
        sendfile_url_for_file.connect(_sendfile_listener, sender=None)

        # we should have gotten a page that says "click here".
        # hit the pd_url again; we should get a sendfile redirect.
        response = self.client.get(self.pd_url)

        self.assertEqual(response['X-Accel-Redirect'], exp_url)
        self.assertEqual(response['X-Sendfile'], exp_url)
        self.assertEqual(response['X-LIGHTTPD-send-file'], exp_url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import csv
import shelve
import feedcache
import socket
#socket.setdefaulttimeout(10)
from xml.sax._exceptions import SAXException

_MEMSTORE='/var/db/feeds/feed_cache'

feed_store = shelve.open(_MEMSTORE)

__all__ = ['get_feed_choices', 'get_feed', 'feed_store']

def get_feed_choices(delimiter=';'):
    """
    Parses a CSV formatted file and returns a dictionary mapping of
    valid feed choices. -> (key_id, url)
    """
    data = {}
    # first get the csv data
    fixture_dir = os.path.join(
        os.path.dirname(__file__), 'fixtures')
    fp = open(os.path.join(fixture_dir, 'feeds.txt'))
    reader = csv.reader(fp, quoting=csv.QUOTE_NONE, delimiter=delimiter)
    for row in reader:
        data[row[1]] = row[0]
    
    return data


def get_feed(feed_id, limit=10):
    


    feed_store['feed_choices'] = get_feed_choices()
    parser = feedcache.Cache(feed_store)

    try:
        feed_choices = feed_store['feed_choices']
        feed = parser.fetch(feed_choices[feed_id], force_update=True)

    except (SAXException, KeyError, socket.error):
        # Invalid feed
        # print 'buggy feed detected: %s' % feed_id
        raise
    
    # Never fetch more than MAX_ITEMS entries
    feed.entries = [item for item in feed.entries][:limit]
    return feed


if __name__ == '__main__':
    lst = get_feed_choices()
    print "%d valid feeds found" % len(lst)
    print lst
    



from django.core.urlresolvers import NoReverseMatch, reverse
from livesettings import config_value
import logging
import re

log = logging.getLogger('recentlist.middleware')

try:
    producturl = reverse('satchmo_product',
        kwargs={'product_slug': 'FAKE'})
    producturl = "^" + producturl.replace("FAKE", r'(?P<slug>[-\w]+)')
    log.debug('Product url is %s', producturl)
    urlre = re.compile(producturl)
except NoReverseMatch:
    log.debug("Could not find product url.")
    urlre = None

class RecentProductMiddleware(object):
    """Remember recent products"""
    def process_response(self, request, response):
        if urlre is not None: # If the product url was found earlier.
            g = urlre.search(request.path)
            if g and len(g.groups()) > 0 and "/admin/" not in request.path:
                recentmax = config_value('PRODUCT','RECENT_MAX') + 1
                slug = g.groups()[0]
                recent = request.session.get('RECENTLIST', [])
                if slug not in recent:
                    recent.insert(0, slug)
                    if len(recent) > recentmax:
                        recent = recent[:recentmax]
                    log.debug('Added recently viewed: %s', recent)
                    request.session['RECENTLIST'] = recent

        return response



from django.core.management.base import BaseCommand
import os
from datetime import date
from tax.modules.us_sst.models import TaxRate
from decimal import Decimal

class Command(BaseCommand):
    '''Manage command to import one of the CSV files from the SST website.

    To update: Simple re-run on the newer CSV file.
    Any unchanged entries will be left alone, and any changed ones will get
    their end dates set properly and the new rows inserted. You will need to do
    this quartly or as-needed by your tax jurisdictions.'''

    def handle(self, *args, **options):
        new = 0
        updated = 0
        unchanged = 0
        file = args[0]
        if not os.path.isfile(file):
            raise RuntimeError("File: %s is not a normal file or doesn't exist." % file)
        file = open(file)
        for line in file:
            (state, type, code, rate_intra, rate_inter, food_intra, food_inter,
             start, end) = line.split(',')
            state = int(state)
            start = date(int(start[0:4]), int(start[4:6]), int(start[6:9]))
            end = date(int(end[0:4]), int(end[4:6]), int(end[6:9]))
            try:
                tr = TaxRate.objects.get(
                    state=state,
                    jurisdictionType=type,
                    jurisdictionFipsCode=code,
                    startDate=start,
                )
                # Over time, end dates can change. A new row with a new start
                # date will also appear. This way, loading a new file correctly
                # updates the map. (I hope.)
                if tr.endDate != end:
                    tr.endDate = end
                    tr.save()
                    updated += 1
                else:
                    unchanged += 1

            except TaxRate.DoesNotExist:
                TaxRate(
                    state=state,
                    jurisdictionType=type,
                    jurisdictionFipsCode=code,
                    startDate=start,
                    endDate=end,
                    generalRateIntrastate = Decimal(rate_intra),
                    generalRateInterstate = Decimal(rate_inter),
                    foodRateIntrastate = Decimal(food_intra),
                    foodRateInterstate = Decimal(food_inter),
                ).save()
                new += 1
        print "Done: New: %d. End date changed: %d. Unchanged: %d" % (new, updated, unchanged)



#!/usr/bin/env python

from moviereviews import global_conf
from pkg_resources import resource_filename

from notmm.utils.django_settings import LazySettings

app_conf = global_conf['moviereviews']

# init the settings module (thread-local)
settings = LazySettings() #SettingsProxy(autoload=True).get_settings()

class DatabaseBackend(object):
    pass

class MySQLBackend(DatabaseBackend):
    from sqlalchemy import create_engine, MetaData
    from sqlalchemy.orm import scoped_session, sessionmaker
    # SQLAlchemy settings
    backend = app_conf['backend']
    schema = backend + ':///' + resource_filename('sandbox', app_conf['path'])
    #engine = sqlalchemy.create_engine(schema, echo=app_conf['echo'])
    engine = create_engine(schema, echo=True)
    metadata = MetaData(engine)
    Session = scoped_session(sessionmaker(bind=engine))




from django.core.management.base import NoArgsCommand

class Command(NoArgsCommand):
    help = ("Invokes recurring billing system to do stuff like "
            "charge subscription customers each month.  You typically "
            "want to invoke this from a cron script.  For non-root "
            "users this is generally done with ``crontab -e``.")

    def handle_noargs(self, **options):
        from payment.views.cron import cron_rebill
        cron_rebill()



"""Product queries using ratings."""
from django.contrib.comments.models import Comment
from django.contrib.sites.models import Site
from keyedcache import cache_get, cache_set, NotCachedError
from livesettings import config_value
from product.models import Product
from satchmo_ext.productratings.utils import average
import logging
import math

log = logging.getLogger('product.comments.queries')

def highest_rated(count=0, site=None):
    """Get the most highly rated products"""
    if site is None:
        site = Site.objects.get_current()

    site_id = site.id

    try:
        pks = cache_get("BESTRATED", site=site_id, count=count)
        pks = [pk for pk in pks.split(',')]
        log.debug('retrieved highest rated products from cache')
        
    except NotCachedError, nce:
        # here were are going to do just one lookup for all product comments

        comments = Comment.objects.filter(content_type__app_label__exact='product',
            content_type__model__exact='product',
            site__id__exact=site_id,
            productrating__rating__gt=0,
            is_public__exact=True).order_by('object_pk')
        
        # then make lists of ratings for each
        commentdict = {}
        for comment in comments:
            if hasattr(comment, 'productrating'):
                rating = comment.productrating.rating
                if rating>0:
                    commentdict.setdefault(comment.object_pk, []).append(rating)
        
        # now take the average of each, and make a nice list suitable for sorting
        ratelist = [(average(ratings), int(pk)) for pk, ratings in commentdict.items()]
        ratelist.sort()
        #log.debug(ratelist)
        
        # chop off the highest and reverse so highest is the first
        ratelist = ratelist[-count:]
        ratelist.reverse()

        pks = ["%i" % p[1] for p in ratelist]
        pkstring = ",".join(pks)
        log.debug('calculated highest rated products, set to cache: %s', pkstring)
        cache_set(nce.key, value=pkstring)
    
    if pks:
        pks = [pk for pk in pks if _int_or_long(pk)]
        productdict = Product.objects.in_bulk(pks)
        products = []
        for pk in pks:
            try:
                if (int(pk)) in productdict:
                    key = int(pk)
                elif long(pk) in productdict:
                    key = long(pk)
                else:
                    continue
                products.append(productdict[key])
            except ValueError:
                pass
    else:
        products = []
        
    return products
        
        
def _int_or_long(v):
    try:
        v = int(v)
    except ValueError:
        try:
            v = long(v)
        except ValueError:
            return False
    return True


'''
Created on 3 Mar 2009

@author: dalore
'''
from django.utils.translation import ugettext as _
from livesettings import config_get_group
from payment.utils import get_processor_by_key
from satchmo_store.shop.models import Cart, Order, OrderPayment
import re

def find_order(data):
    """
    Helper function to find order using a google id
    """
    transaction_id = data['google-order-number']
    payment = OrderPayment.objects.filter(transaction_id__exact=transaction_id)[0]
    order = payment.order
    return order

def notify_neworder(request, data):
    """
    Called when google reports a new order.
    
    Looks up the order from the private data and sets the status.
    Empties the cart.
    """
    # get params from data
    private_data = data['shopping-cart.merchant-private-data']
    order_id = re.search('satchmo-order id="(\d+)"', private_data).group(1)
    order = Order.objects.get(pk=order_id)
    payment_module = config_get_group('PAYMENT_GOOGLE')
    processor = get_processor_by_key('PAYMENT_GOOGLE')
    
    # record pending payment
    amount = data['order-total']
    pending_payment = processor.create_pending_payment(order)
    # save transaction id so we can find this order later
    pending_payment.capture.transaction_id = data['google-order-number']
    pending_payment.capture.save()
    
    # delete cart
    for cart in Cart.objects.filter(customer=order.contact):
        cart.empty()
        cart.delete()
        
    # set status
    order.add_status(status='New', notes=_("Received through Google Checkout."))
    
        
def do_charged(request, data):
    """
    Called when google sends a charged status update
    Note that the charged amount comes in a seperate call
    """
    # find order from google id
    order = find_order(data)
    
    # Added to track total sold for each product
    for item in order.orderitem_set.all():
        product = item.product
        product.total_sold += item.quantity
        product.items_in_stock -= item.quantity
        product.save()
        
    # process payment
    processor = get_processor_by_key('PAYMENT_GOOGLE')
    # setting status to billed (why does paypal set it to new?)
    order.add_status(status='Billed', notes=_("Paid through Google Checkout."))
    
def do_shipped(request, data):
    """
    Called when you use the google checkout console to mark order has been shipped
    """
    # find order from google id
    order = find_order(data)
    # process payment
    processor = get_processor_by_key('PAYMENT_GOOGLE')
    # setting status to billed (why does paypal set it to new?)
    order.add_status(status='Shipped', notes=_("Shipped through Google Checkout."))
    

def notify_statechanged(request, data):
    """
    This is called when there has been a change in the order state
    """
    # financial state
    financial_state = data['new-financial-order-state']
    fulfillment_state = data['new-fulfillment-order-state']
    if financial_state == 'CHARGED':
        if fulfillment_state == 'PROCESSING':
            do_charged(request, data)
        elif fulfillment_state == 'DELIVERED':
            do_shipped(request, data)
    
        
def notify_chargeamount(request, data):
    """
    This gets called when google sends a charge amount
    """
    # find order from google id
    order = find_order(data)
    transaction_id = data['google-order-number']
    processor = get_processor_by_key('PAYMENT_GOOGLE')
    processor.record_payment(amount=data['latest-charge-amount'], transaction_id=transaction_id, order=order)




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) 2007-2019 Etienne Robillard <tkadm30@yandex.com>
# All rights reserved.
"""WSGI helper functions and classes for development purposes.

"""
import sys
import urllib
import logging
import time
import platform
python_impl = platform.python_implementation()

from wsgiref import simple_server, validate

from notmm.utils.configparse import string_getter, int_getter
from notmm.utils.wsgilib import translogger 
from notmm.release import BASEVERSION, MINOR_REVISION
from django.utils.termcolors import colorize as _colored
    
from .reloader import install

log = logging.getLogger(__name__)

__all__ = ('WSGIServerBase', 'BannerBase', 'get_bind_addr', 'daemonize')

class BannerBase(object):
    """This is a simple startup banner.

    Usage:
        >>> banner = BannerBase(fp=sys.stderr, ('127.0.0.1', '8000'))
        >>> banner.show()
    """

    default_format = "%Y-%m-%d %H:%M:%S %Z %z"
    debug_msg_handler = log.info

    def __init__(self, fp=debug_msg_handler, bind_addr=('127.0.0.1', '8000'),
                 settings=None):
        self.bind_addr = bind_addr
        fmt = self.get_default_format(settings)
        self.start_time = time.strftime(fmt)
        self.server_uri = 'http://%s:%i/' % bind_addr
        if not isinstance(self.bind_addr, tuple):
            raise ValueError("Error: invalid bind_addr type: %r" \
                % type(self.bind_addr))

        #assert len(self.bind_addr) == 2, \
        #    'bind_addr must contains exactly two elements'

    def get_default_format(self, settings):
        try:
            fmt = settings.DATETIME_INPUT_FORMATS[0]
        except (AttributeError, IndexError):
            return self.default_format
        else:
            return fmt
    
    def debug(self, message, color='cyan', **kwargs):
        """Write a string to the terminal with colored output"""
        self.debug_msg_handler(_colored(message, color, **kwargs))

    def show(self):
        """Override this method to handle the formatting of the banner"""

        self.debug("Initializing on %s" % self.start_time)
        self.debug("Starting HTTP server on %s" % self.server_uri)
        #colored("notmm %s "%(notmm.__version__)) 
        #colored("%s"%notmm.__copyright__)
        # XXX this part need more work ! :)
        self.debug("Django-hotsauce release %s revision %s (%s)" % (
            BASEVERSION, MINOR_REVISION, python_impl
            ))
        return self

class WSGIServerBase(object):
    """Wrapper class to configure a simple HTTP server instance"""
    
    # Uncomment this to enable cherrypy based http daemon
    # serverClass = wsgiserver.CherryPyWSGIServer
    
    serverClass = simple_server # wsgiref
    
    def __init__(self, wsgi_app, bind_addr, debug=True):
        """HTTPServer.__init__"""
        self.host = bind_addr[0]
        self.port = bind_addr[1]
        self.wsgi_app = wsgi_app
        if debug:
            # run the wsgi app in using wsgiref validator
            # middleware
            self.request_handler = validate.validator(wsgi_app)
        else:
            self.request_handler = wsgi_app

        #
        b = BannerBase(bind_addr=bind_addr)
        b.show()
    
        self.server = self.serverClass.make_server(self.host, self.port, self.request_handler)

    def serve(self):
        try:
            return self.server.serve_forever()
        except:
            raise

def get_bind_addr(app_conf, section='httpserver', listen_port=8000):
    """
    Return a 2-element tuple containing the hostname and port
    to listen for remote connections.
    """

    host = string_getter(app_conf, section, 'host') or urllib.localhost()
    port = int_getter(app_conf, section, 'port') or listen_port

    return (host, port)

def daemonize(wsgi_app, bind_addr, logging=False, autoreload=True, debug=True):
    """
    Create an instance of a validating WSGI server for 
    development purposes.
    
    XXX: enabling WSGI validation (debug=True) may trigger
    a deadlock for POST input data processing.
    """
    
    # Install translogger middleware if logging is enabled
    if logging:
        request_handler = translogger.TransLogger(wsgi_app)
    else:
        request_handler = wsgi_app
    
    # Add optional autoreloading support (see the docstring for help
    # on installing this)
    if autoreload:
        try:
            install()
        except ImportError:
            pass


    # init the wsgi server 
    server = WSGIServerBase(request_handler, bind_addr, debug=debug)

    try:
        server.serve()
    except (KeyboardInterrupt, SystemExit):
        sys.exit(0)
    except:
        raise

class HTTPServer(WSGIServerBase):
    pass



from django.shortcuts import render_to_response
from django.template import RequestContext
from livesettings import config_value
from satchmo_ext.productratings.queries import highest_rated

def display_bestratings(request, count=0, template='product/best_ratings.html'):
    """Display a list of the products with the best ratings in comments"""
    if count is None:
        count = config_value('PRODUCT','NUM_DISPLAY')
    
    ctx = RequestContext(request, {
        'products' : highest_rated(),
    })
    return render_to_response(template, context_instance=ctx)



"""
Tiered shipping models
"""
from decimal import Decimal
from django.conf import settings
from django.db import models
from django.utils.translation import get_language, ugettext_lazy as _
from shipping.modules.base import BaseShipper
import datetime
import logging
import operator

log = logging.getLogger('shipping.TieredQuantity')

class TieredPriceException(Exception):
    def __init__(self, reason):
        self.reason = reason

class Shipper(BaseShipper):

    def __init__(self, carrier):
        self.id = carrier.key
        self.carrier = carrier
        super(BaseShipper, self).__init__()

    def __str__(self):
        """
        This is mainly helpful for debugging purposes
        """
        return "Tiered_Shipper: %s" % self.id

    def description(self):
        """
        A basic description that will be displayed to the user when selecting their shipping options
        """
        return self.carrier.description

    def cost(self):
        """
        Complex calculations can be done here as long as the return value is a dollar figure
        """
        assert(self._calculated)
        qty = Decimal('0')
        for cartitem in self.cart.cartitem_set.all():
            if cartitem.product.is_shippable:
                qty += cartitem.quantity
        return self.carrier.price(qty)

    def method(self):
        """
        Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        """
        return self.carrier.method

    def expectedDelivery(self):
        """
        Can be a plain string or complex calcuation returning an actual date
        """
        return self.carrier.delivery

    def valid(self, order=None):
        """
        Check to see if this order can be shipped using this method
        """
        if order:
            quants = [item.quantity for item in order.orderitem_set.all() if item.product.is_shippable]
            if quants:
                qty = reduce(operator.add, quants)
            else:
                qty = Decimal('0')
                                                
        elif self.cart:
            qty = self.cart.numItems

        try:
            price = self.carrier.price(qty)
            
        except TieredPriceException:
            return False
                        
        return True


class Carrier(models.Model):
    key = models.SlugField(_('Key'))
    ordering = models.IntegerField(_('Ordering'), default=0)
    active = models.BooleanField(_('Active'), default=True)

    def _find_translation(self, language_code=None):
        if not language_code:
            language_code = get_language()
            
        c = self.translations.filter(languagecode__exact = language_code)
        ct = c.count()

        if not c or ct == 0:
            pos = language_code.find('-')
            if pos>-1:
                short_code = language_code[:pos]
                log.debug("%s: Trying to find root language content for: [%s]", self.id, short_code)
                c = self.translations.filter(languagecode__exact = short_code)
                ct = c.count()
                if ct>0:
                    log.debug("%s: Found root language content for: [%s]", self.id, short_code)

        if not c or ct == 0:
            #log.debug("Trying to find default language content for: %s", self)
            c = self.translations.filter(languagecode__istartswith = settings.LANGUAGE_CODE)
            ct = c.count()

        if not c or ct == 0:
            #log.debug("Trying to find *any* language content for: %s", self)
            c = self.translations.all()
            ct = c.count()

        if ct > 0:
            trans = c[0]
        else:
            trans = None

        return trans

    def delivery(self):
        """Get the delivery, looking up by language code, falling back intelligently.
        """
        trans = self._find_translation()

        if trans:
            return trans.delivery
        else:
            return ""

    delivery = property(delivery)
 
    def description(self):
        """Get the description, looking up by language code, falling back intelligently.
        """
        trans = self._find_translation()

        if trans:
            return trans.description
        else:
            return ""

    description = property(description)
    
    def method(self):
        """Get the description, looking up by language code, falling back intelligently.
        """
        trans = self._find_translation()

        if trans:
            return trans.method
        else:
            return ""

    method = property(method)    
 
    def name(self):
        """Get the name, looking up by language code, falling back intelligently.
        """
        trans = self._find_translation()

        if trans:
            return trans.name
        else:
            return self.key

    name = property(name)
    
    def price(self, qty):
        """Get a price for this qty."""
        # first check for special discounts
        prices = self.tiers.filter(expires__isnull=False, quantity__lte=qty).exclude(expires__lt=datetime.date.today())
        
        if not prices.count() > 0:
            prices = self.tiers.filter(expires__isnull=True, quantity__lte=qty)

        if prices.count() > 0:
            # Get the price with the quantity closest to the one specified without going over
            return Decimal(prices.order_by('-quantity')[0].calculate_price(qty))

        else:
            log.debug("No quantity tier found for %s: qty=%d", self.id, qty)
            raise TieredPriceException('No price available')
            
            
    def __unicode__(self):
        return u"Carrier: %s" % self.name
        
    class Admin:
        ordering = ('key',)

    class Meta:
        pass
        
class CarrierTranslation(models.Model):
    carrier = models.ForeignKey('Carrier', related_name='translations')
    languagecode = models.CharField(_('language'), max_length=10, choices=settings.LANGUAGES, )
    name = models.CharField(_('Carrier'), max_length=50, )
    description = models.CharField(_('Description'), max_length=200)
    method = models.CharField(_('Method'), help_text=_("i.e. US Mail"), max_length=200)
    delivery = models.CharField(_('Delivery Days'), max_length=200)

class QuantityTier(models.Model):
    carrier = models.ForeignKey('Carrier', related_name='tiers')
    quantity = models.DecimalField(_("Min Quantity"), max_digits=18,  decimal_places=6,
        help_text=_('Minimum qty in order for this to apply?'), )
    handling = models.DecimalField(_("Handling Price"), max_digits=10, 
        decimal_places=2, )
    price = models.DecimalField(_("Shipping Per Item"), max_digits=10, 
        decimal_places=2, )
    expires = models.DateField(_("Expires"), null=True, blank=True)
    
    def calculate_price(self, qty):
        return self.handling + self.price * qty
    
    def __unicode__(self):
        return u"QuantityTier: %s @ %s" % (self.price, self.quantity)
    
    class Admin:
        ordering = ('min_total', 'expires')
    
    class Meta:
        pass

import config
